@@ -17,6 +17,7 @@ import os.path
 import gettext
 import urllib # To decode URI's
 import Globals # To get projectfolder
+import ui.EventLaneHSeparator as EventLaneHSeparator
 _ = gettext.gettext
 
 #=========================================================================
@@ -58,9 +59,9 @@ class EventLaneViewer(gtk.EventBox):
 		self.vbox = gtk.VBox()
 		self.fixed = gtk.Fixed()
 
-		self.separator = gtk.HSeparator()
+		self.separator = EventLaneHSeparator.EventLaneHSeparator(project, project.transport)
 		self.vbox.pack_start(self.fixed, True, True)
-		self.vbox.pack_end(self.separator, False, True)
+		self.vbox.pack_start(self.separator, False, False)
 
 		self.vbox.show_all()
 		self.add(self.vbox)
@@ -148,24 +149,12 @@ class EventLaneViewer(gtk.EventBox):
 		
 		transport = self.project.transport
 		
-		# Draw lane edges
-		col = gc.get_colormap().alloc_color("#666666")
-		gc.set_foreground(col)
-		wnd.draw_line(gc, 0, self.allocation.height-1, self.allocation.width-1, self.allocation.height-1)
-		
 		# Draw play cursor position
 		col = gc.get_colormap().alloc_color("#FF0000")
 		gc.set_foreground(col)
-		
-		x = int(round((transport.position - self.project.viewStart) * self.project.viewScale))
+		x = transport.GetPixelPosition()
 		wnd.draw_line(gc, x, 0, x, self.allocation.height)
 		
-		# Draw edit position
-		if self.highlightCursor:
-			col = gc.get_colormap().alloc_color("#0000FF")
-			gc.set_foreground(col)
-			wnd.draw_line(gc, int(self.highlightCursor), 0, int(self.highlightCursor), self.allocation.height)
-		
 	#_____________________________________________________________________
 		
 	def UpdatePosition(self, eventViewer):
@@ -411,10 +400,10 @@ class EventLaneViewer(gtk.EventBox):
 			extraString -- a string specifying the extra action details. i.e. "stop-action"
 					means that the position changed because the user hit stop.
 		"""
-		x1 = round((self.project.transport.PrevPosition - self.project.viewStart) * self.project.viewScale)
-		x2 = round((self.project.transport.position - self.project.viewStart) * self.project.viewScale)
-		self.queue_draw_area(int(x1)-1, 0, 3, self.allocation.height)
-		self.queue_draw_area(int(x2)-1, 0, 3, self.allocation.height)
+		prev_pos = self.project.transport.GetPreviousPixelPosition()
+		new_pos = self.project.transport.GetPixelPosition()
+		self.queue_draw_area(prev_pos - 1, 0, 3, self.allocation.height)
+		self.queue_draw_area(new_pos - 1, 0, 3, self.allocation.height)
 	
 	#_____________________________________________________________________
 	
