@@ -49,9 +49,9 @@ class EventViewer(gtk.DrawingArea):
 	   RGBA = Red, Green, Blue, Alpha
 	   RGB = Red, Green, Blue
 	"""
-	_OPAQUE_GRADIENT_STOP_ORGBA = (0.2, 138./255, 226./255, 52./255, 1)
-	_TRANSPARENT_GRADIENT_STOP_ORGBA = (1, 138./255, 226./255, 52./255, 0.5)
-	_BORDER_RGB = (79./255, 154./255, 6./255)
+	_OPAQUE_GRADIENT_STOP_ORGBA = (0.2, 114./255, 159./255, 207./255, 1)
+	_TRANSPARENT_GRADIENT_STOP_ORGBA = (1, 52./255, 101./255, 164./255, 1)
+	_BORDER_RGB = (32./255, 74./255, 135./255) 
 	_BORDER_HIGHLIGHT_RGB = (0, 0, 1)
 	_BACKGROUND_RGB = (1, 1, 1)
 	_TEXT_RGB = (0, 0, 0)
@@ -239,11 +239,13 @@ class EventViewer(gtk.DrawingArea):
 		context.set_line_width(2)
 		
 		#Draw play position
-		x = int(round((self.project.transport.position - self.event.start) * self.project.viewScale))
+		# TODO: don't calculate pixel position based on self.event.start, it will have rounding errros
+		# instead determine the pixel position of the start of our widget and subtract that from GetPixelPosition().
+		x = self.project.transport.GetPixelPosition(self.event.start)
 		context.set_line_width(1)
 		context.set_antialias(cairo.ANTIALIAS_NONE)
-		context.move_to(x+1, 0)
-		context.line_to(x+1, self.allocation.height)
+		context.move_to(x+0.5, 0)
+		context.line_to(x+0.5, self.allocation.height)
 		context.set_source_rgb(*self._PLAY_POSITION_RGB)
 		context.stroke()
 		
@@ -832,7 +834,7 @@ class EventViewer(gtk.DrawingArea):
 				self.OnSplit(None, self.highlightCursor)
 			else:
 				# Otherwise, stop playing and cut at the play position (if it's over this event)
-				play_pos = int(round((self.project.transport.position - self.event.start) * self.project.viewScale))
+				play_pos = self.project.transport.GetPixelPosition(self.event.start)
 				if play_pos > 0 and play_pos < self.allocation.width:
 					self.project.Stop()
 					self.OnSplit(None, play_pos)
