@@ -6,7 +6,7 @@
 #	
 #	This module contains variable definitions that can be used across the code
 #	base and also includes methods for reading and writing these settings to
-#	the Jokosher configuration in ~/.jokosher/config.
+#	the Jokosher configuration in JOKOSHER_CONFIG_HOME/config.
 #
 #-------------------------------------------------------------------------------
 
@@ -16,6 +16,8 @@ import locale, gettext
 import pygtk
 pygtk.require("2.0")
 import gobject, gtk
+import xdg.BaseDirectory
+import shutil
 
 import gettext
 _ = gettext.gettext
@@ -27,47 +29,46 @@ class Settings:
 
 	# the different settings in each config block
 	general = 	{
+				# increment each time there is an incompatible change with the config file
+				"version" : "1",
 				"recentprojects": "value", 
 				"startupaction" : "value",
 				"projectfolder" : "",
 				"windowheight" : 550,
 				"windowwidth" : 900,
-				"backupsavetime" : 30000,
-				"lastbackup" : 0
 				}
 	
 	recording = {
 				"fileformat": "flacenc",
 				"file_extension": "flac",
-				"samplerate": "0" # zero means, autodetect sample rate (ie use any available)
+				"samplerate": "0", # zero means, autodetect sample rate (ie use any available)
+				"audiosrc" : "gconfaudiosrc",
+				"device" : "default"
 				}
 	
 	playback = 	{
+				"devicename": "default",
 				"device": "default",
-				"devicecardnum": "default",
 				"audiosink":"autoaudiosink"
 				}
 	
 	extensions = {
 				 "extensions_blacklist": ""
 				 }
+				 
+	sections = {
+				"General" : general,
+				"Recording" : recording,
+				"Playback" : playback,
+				"Extensions" : extensions
+				}
 
 	#_____________________________________________________________________
 	
-	def __init__(self, filename = None):
-		"""
-		Creates a new instance of Settings.
+	def __init__(self):
+		self.filename = os.path.join(JOKOSHER_CONFIG_HOME, "config")
 		
-		Parameters:
-			filename -- path to the settings file.
-						If None, the default ~/.jokosher/config will be used.
-		"""
-		if not filename:
-			self.filename = os.path.expanduser("~/.jokosher/config")
-		else:
-			self.filename = filename
 		self.config = ConfigParser.ConfigParser()
-
 		self.read()
 
 	#_____________________________________________________________________
@@ -79,48 +80,27 @@ class Settings:
 		"""
 		self.config.read(self.filename)
 	
-		if not self.config.has_section("General"):
-			self.config.add_section("General")
-		if not self.config.has_section("Recording"):
-			self.config.add_section("Recording")
-		if not self.config.has_section("Playback"):
-			self.config.add_section("Playback")
-		if not self.config.has_section("Extensions"):
-			self.config.add_section("Extensions")
+		for section in self.sections:
+			if not self.config.has_section(section):
+				self.config.add_section(section)
 	
-		for key, value in self.config.items("General"):
-			self.general[key] = value
-		for key, value in self.config.items("Recording"):
-			self.recording[key] = value
-		for key, value in self.config.items("Playback"):
-			self.playback[key] = value
-		for key, value in self.config.items("Extensions"):
-			self.extensions[key] = value
+		for section, section_dict in self.sections.iteritems():
+			for key, value in self.config.items(section):
+				if value == "None":
+					value = None
+				section_dict[key] = value
 	
 	#_____________________________________________________________________
 		
 	def write(self):
 		"""
 		Writes configuration settings to the Settings config file.
-		"""		
-		for key in self.general:
-			self.config.set("General", key, self.general[key])
-		for key in self.recording:
-			self.config.set("Recording", key, self.recording[key])
-		for key in self.playback:
-			self.config.set("Playback", key, self.playback[key])
-		for key in self.extensions:
-			self.config.set("Extensions", key, self.extensions[key])
-			
-		# delete a .jokosher file if it exists, because that's old-fashioned
-		old_jokosher_file = os.path.expanduser("~/.jokosher")
-		if os.path.isfile(old_jokosher_file):
-			os.unlink(old_jokosher_file)
+		"""
+
+		for section, section_dict in self.sections.iteritems():
+			for key, value in section_dict.iteritems():
+				self.config.set(section, key, value)
 		
-		# make sure that the directory that the config file is in exists
-		new_jokosher_file_dir = os.path.split(self.filename)[0]
-		if not os.path.isdir(new_jokosher_file_dir): 
-			os.makedirs(new_jokosher_file_dir)
 		file = open(self.filename, 'w')
 		self.config.write(file)
 		file.close()
@@ -144,7 +124,7 @@ def debug(*listToPrint):
 	message = " ".join( [ str(x) for x in listToPrint ] )
 	
 	if DEBUG_STDOUT:
-		print message
+		print(message)
 	if DEBUG_GST:
 		gst.debug(message)
 		
@@ -283,6 +263,14 @@ def getCachedInstruments(checkForNew=False):
 
 #_____________________________________________________________________
 
+def getCachedInstrumentPixbuf(get_type):
+	for (name, type, pixbuf, pixbufPath) in getCachedInstruments():
+		if type == get_type:
+			return pixbuf
+	return None
+
+#_____________________________________________________________________
+
 def idleCacheInstruments():
 	"""
 	Loads the Instruments 'lazily' to avoid blocking the GUI.
@@ -318,27 +306,66 @@ def PopulateEncoders():
 	"""
 	Check if the hardcoded list of encoders is available on the system.
 	"""
+	for type in _export_formats:
+		if VerifyAllElements(type[2]):
+			#create a dictionary using _export_template as the keys
+			#and the current item from _export_formats as the values.
+			d = dict(zip(_export_template, type))
+			EXPORT_FORMATS.append(d)
+
+#_____________________________________________________________________
+
+def VerifyAllElements(bin_desc):
 	#HACK: we can't import gst at the top of Globals.py because
 	#if we do, gstreamer will get to the sys.args and print it's own
 	#message instead of ours. This will be fixed once we can use
 	#GOption when we depend on pygobject 2.12.
 	import gst
 	
-	for type in _export_formats:
-		all_elements_exist = True
-		for element in type[2].split("!"):
-			exists = gst.default_registry_check_feature_version(element.strip(), 0, 10, 0)
-			if not exists:
-				all_elements_exist = False
-				debug('Cannot find "%s" plugin, disabling encoder: "%s"' % (element.strip(), type[2]))
-				# we know at least one of the elements doesnt exist, so skip this encode format.
-				break
+	all_elements_exist = True
+	for element in bin_desc.split("!"):
+		exists = gst.default_registry_check_feature_version(element.strip(), 0, 10, 0)
+		if not exists:
+			all_elements_exist = False
+			debug('Cannot find "%s" plugin, disabling: "%s"' % (element.strip(), bin_desc))
+			# we know at least one of the elements doesnt exist, so skip this encode format.
+			break
+	
+	return all_elements_exist
+	
+#_____________________________________________________________________
+
+def PopulateAudioBackends():
+	CheckBackendList(PLAYBACK_BACKENDS)
+	CheckBackendList(CAPTURE_BACKENDS)
+	
+#_____________________________________________________________________
+	
+def CheckBackendList(backend_list):
+	remove_list = []
+	for tuple_ in backend_list:
+		bin_desc = tuple_[1]
+		if not VerifyAllElements(bin_desc):
+			remove_list.append(tuple_)
+	
+	for tuple_ in remove_list:
+		backend_list.remove(tuple_)
+
+#_____________________________________________________________________
+
+def CopyAllFiles(src_dir, dest_dir, only_these_files=None):
+	""" Copies all the files, but only the files from one directory to another."""
+	for file in os.listdir(src_dir):
+		if only_these_files and file not in only_these_files:
+			continue
 		
-		if all_elements_exist:
-			#create a dictionary using _export_template as the keys
-			#and the current item from _export_formats as the values.
-			d = dict(zip(_export_template, type))
-			EXPORT_FORMATS.append(d)
+		src_path = os.path.join(src_dir, file)
+		dest_path = os.path.join(dest_dir, file)
+		if os.path.isfile(src_path):
+			try:
+				shutil.copy2(src_path, dest_path)
+			except IOError:
+				print "Unable to copy from old ~/.jokosher directory:", src_path
 
 #_____________________________________________________________________
 
@@ -356,39 +383,66 @@ Global paths, so all methods can access them.
 If JOKOSHER_DATA_PATH is not set, that is, Jokosher is running locally,
 use paths relative to the current running directory instead of /usr ones.
 """
+
+XDG_RESOURCE_NAME = "jokosher"
+JOKOSHER_CONFIG_HOME = xdg.BaseDirectory.save_config_path(XDG_RESOURCE_NAME)
+JOKOSHER_DATA_HOME =   xdg.BaseDirectory.save_data_path(XDG_RESOURCE_NAME)
+
 data_path = os.getenv("JOKOSHER_DATA_PATH")
 if data_path:
-	INSTR_PATHS = (os.path.join(data_path, "Instruments"), os.path.expanduser("~/.jokosher/instruments"))
-	EXTENSION_PATHS = (os.path.join(data_path, "extensions"), os.path.expanduser("~/.jokosher/extensions/"))
+	INSTR_PATHS = (os.path.join(data_path, "Instruments"), os.path.join(JOKOSHER_DATA_HOME, "instruments"))
+	EXTENSION_PATHS = (os.path.join(data_path, "extensions"), os.path.join(JOKOSHER_DATA_HOME, "extensions"))
 	GLADE_PATH = os.path.join(data_path, "Jokosher.glade")
 else:
 	data_path = os.path.dirname(os.path.abspath(__file__))
-	INSTR_PATHS = (os.path.join(data_path, "..", "Instruments"), os.path.expanduser("~/.jokosher/instruments"))
-	EXTENSION_PATHS = (os.path.join(data_path, "..", "extensions"), os.path.expanduser("~/.jokosher/extensions/"))
+	INSTR_PATHS = (os.path.join(data_path, "..", "Instruments"), os.path.join(JOKOSHER_DATA_HOME, "instruments"))
+	EXTENSION_PATHS = (os.path.join(data_path, "..", "extensions"), os.path.join(JOKOSHER_DATA_HOME, "extensions"))
 	GLADE_PATH = os.path.join(data_path, "Jokosher.glade")
 	LOCALE_PATH = os.path.join(data_path, "..", "locale")
 
-#delete the 0.1 jokosher config file
-if os.path.isfile(os.path.expanduser("~/.jokosher")):
-	try:
-		os.remove(os.path.expanduser("~/.jokosher"))
-	except:
-		raise "Failed to delete old user config file %s" % new_dir
 # create a couple dirs to avoid having problems creating a non-existing
 # directory inside another non-existing directory
-for directory in ['extensions', 'instruments', 'instruments/images', 
-		'presets', 'presets/effects', 'presets/mixdown', 'mixdownprofiles', 'templates']:
-	new_dir = os.path.join(os.path.expanduser("~/.jokosher/"), directory)
+create_dirs = [
+	'extensions',
+	'instruments',
+	('instruments', 'images'),
+	'presets',
+	('presets', 'effects'),
+	('presets', 'mixdown'),
+	'mixdownprofiles',
+	'templates',
+]
+
+# do a listing before we create the dirs so we know if it was empty (ie first run)
+jokosher_dir_empty = (len(os.listdir(JOKOSHER_DATA_HOME)) == 0)
+_HOME_DOT_JOKOSHER = os.path.expanduser("~/.jokosher")
+
+if jokosher_dir_empty and os.path.isdir(_HOME_DOT_JOKOSHER):
+	# Copying old config file from ~/.jokosher.
+	CopyAllFiles(_HOME_DOT_JOKOSHER, JOKOSHER_CONFIG_HOME, ["config"])
+
+for dirs in create_dirs:
+	if isinstance(dirs, str):
+		new_dir = os.path.join(JOKOSHER_DATA_HOME, dirs)
+		old_dir = os.path.join(_HOME_DOT_JOKOSHER, dirs)
+	else:
+		new_dir = os.path.join(JOKOSHER_DATA_HOME, *dirs)
+		old_dir = os.path.join(_HOME_DOT_JOKOSHER, *dirs)
+		
 	if not os.path.isdir(new_dir):
 		try:
 			os.makedirs(new_dir)
 		except:
 			raise "Failed to create user config directory %s" % new_dir
+	
+	if jokosher_dir_empty and os.path.isdir(old_dir) and os.path.isdir(new_dir):
+		CopyAllFiles(old_dir, new_dir)
+			
 
 #TODO: make this a list with the system path and home directory path
-EFFECT_PRESETS_PATH = os.path.expanduser("~/.jokosher/presets/effects")
-TEMPLATES_PATH = os.path.expanduser("~/.jokosher/templates/")
-MIXDOWN_PROFILES_PATH = os.path.expanduser("~/.jokosher/mixdownprofiles/")
+EFFECT_PRESETS_PATH = os.path.join(JOKOSHER_DATA_HOME, "presets", "effects")
+TEMPLATES_PATH = os.path.join(JOKOSHER_DATA_HOME, "templates")
+MIXDOWN_PROFILES_PATH = os.path.join(JOKOSHER_DATA_HOME, "mixdownprofiles")
 
 IMAGE_PATH = os.getenv("JOKOSHER_IMAGE_PATH")
 if not IMAGE_PATH:
@@ -421,7 +475,6 @@ if __extra_ext_dirs:
 
 """ ExtensionManager data """
 AVAILABLE_EXTENSIONS = []
-INSTRUMENT_HEADER_WIDTH = 0
 
 """ Locale constant """
 LOCALE_APP = "jokosher"
@@ -646,6 +699,23 @@ EXPORT_FORMATS = []
 
 SAMPLE_RATES = [8000, 11025, 22050, 32000, 44100, 48000, 96000, 192000]
 
+PLAYBACK_BACKENDS = [
+	(_("Autodetect"), "autoaudiosink"),
+	(_("Use GNOME Settings"), "gconfaudiosink"),
+	("ALSA", "alsasink"),
+	("OSS", "osssink"),
+	("JACK", "jackaudiosink"),
+	("PulseAudio", "pulsesink"),
+]
+
+CAPTURE_BACKENDS = [
+	(_("GNOME Settings"), "gconfaudiosrc"),
+	("ALSA", "alsasrc"),
+	("OSS", "osssrc"),
+	("JACK", "jackaudiosrc"),
+	("PulseAudio", "pulsesrc"),
+]
+
 """ Default Instruments """
 DEFAULT_INSTRUMENTS = []
 """ init Settings """
@@ -654,6 +724,13 @@ settings = Settings()
 """ Cache Instruments """
 gobject.idle_add(idleCacheInstruments)
 
+
+gobject.set_application_name(_("Jokosher Audio Editor"))
+gobject.set_prgname(LOCALE_APP)
+gtk.window_set_default_icon_name("jokosher")
+# environment variable for pulseaudio type
+os.environ["PULSE_PROP_media.role"] = "production"
+
 # I have decided that Globals.py is a boring source file. So, here is a little
 # joke. What does the tax office and a pelican have in common? They can both stick
 # their bills up their arses. Har har har.
