@@ -19,11 +19,10 @@ import os, time, shutil
 import urlparse # To split up URI's
 import gobject
 import Event
-import UndoSystem
+import UndoSystem, IncrementalSave
 import Utils
 
 import Globals
-import AlsaDevices
 import gettext
 _ = gettext.gettext
 
@@ -114,17 +113,12 @@ class Instrument(gobject.GObject):
 		self.volume = 1.0			# Gain of the current instrument in range 0..1
 		self.pan = 0.0				# pan number (between -100 and 100)
 		self.currentchainpreset = None	# current instrument wide chain preset
-		self.inTrack = 0			# Input track to record from
 		self.output = ""
 		self.recordingbin = None
 		self.id = project.GenerateUniqueID(id)	#check is id is already being used before setting
 		
-		# Select first input device as default to avoid a GStreamer bug which causes
-		# large amounts of latency with the ALSA 'default' device.
-		try:
-			self.input = AlsaDevices.GetAlsaList("capture").keys()[1]
-		except: 
-			self.input = "default"
+		self.input = None	# the device to use for recording on this instrument.
+		self.inTrack = -1	# Input track to record from if device is multichannel.
 	
 		# CREATE GSTREAMER ELEMENTS #
 		self.playbackbin = gst.element_factory_make("bin", "Instrument_%d"%self.id)
@@ -498,6 +492,9 @@ class Instrument(gobject.GObject):
 		event.file = os.path.join(self.project.audio_path, filename)
 		event.levels_file = filename + Event.Event.LEVELS_FILE_EXTENSION
 		
+		inc = IncrementalSave.NewEvent(self.id, filename, event.start, event.id, recording=True)
+		self.project.SaveIncrementalAction(inc)
+		
 		#must add it to the instrument's list so that an update of the event lane will not remove the widget
 		self.events.append(event)
 		self.emit("event::added", event)
@@ -505,7 +502,7 @@ class Instrument(gobject.GObject):
 
 	#_____________________________________________________________________
 
-	@UndoSystem.UndoCommand("DeleteEvent", "temp")
+	@UndoSystem.UndoCommand("DeleteEvent", "temp", incremental_save=False)
 	def FinishRecordingEvent(self, event):
 		"""
 		Called to log the adding of this event on the undo stack
@@ -569,8 +566,8 @@ class Instrument(gobject.GObject):
 	
 	#_____________________________________________________________________
 
-	@UndoSystem.UndoCommand("DeleteEvent", "temp")
-	def addEventFromFile(self, start, file, copyfile=False):
+	@UndoSystem.UndoCommand("DeleteEvent", "temp", incremental_save=False)
+	def addEventFromFile(self, start, file, copyfile=False, name=None, duration=None, levels_file=None):
 		"""
 		Adds an Event from a file to this Instrument.
 		
@@ -579,17 +576,19 @@ class Instrument(gobject.GObject):
 			file -- path to the Event file.
 			copyfile --	True = copy the file to Project's audio directory.
 						False = don't copy the file to the Project's audio directory.
+			name -- An optional user visible name. The filename will be used if None.
 						
 		Returns:
 			the added Event.
 		"""
 		filelabel=file
 		event_id = self.project.GenerateUniqueID(None,  reserve=False)
-		name = os.path.basename(file)
+		if not name:
+			name = os.path.basename(file)
 		root,  extension = os.path.splitext(name.replace(" ", "_"))
 		
 		if extension:
-			newfile = "%s_%d.%s" % (root, event_id, extension)
+			newfile = "%s_%d%s" % (root, event_id, extension)
 		else:
 			newfile = "%s_%d" % (root, event_id)
 
@@ -602,14 +601,29 @@ class Instrument(gobject.GObject):
 				raise UndoSystem.CancelUndoCommand()
 				
 			self.project.deleteOnCloseAudioFiles.append(audio_file)
+			inc = IncrementalSave.NewEvent(self.id, newfile, start, event_id)
+			self.project.SaveIncrementalAction(inc)
 			
 			file = audio_file
+		else:
+			inc = IncrementalSave.NewEvent(self.id, file, start, event_id)
+			self.project.SaveIncrementalAction(inc)
 
 		ev = Event.Event(self, file, event_id, filelabel)
 		ev.start = start
 		ev.name = name
 		self.events.append(ev)
-		ev.GenerateWaveform()
+		
+		if duration and levels_file:
+			ev.duration = duration
+			ev.levels_file = levels_file
+			levels_path = os.path.join(self.project.levels_path, levels_file)
+			ev.levels_list.fromfile(levels_path)
+			# update properties and position when duration changes.
+			ev.MoveButDoNotOverlap(ev.start)
+			ev.SetProperties()
+		else:
+			ev.GenerateWaveform()
 
 		self.temp = ev.id
 		
@@ -619,7 +633,7 @@ class Instrument(gobject.GObject):
 		
 	#_____________________________________________________________________
 	
-	@UndoSystem.UndoCommand("DeleteEvent", "temp")
+	@UndoSystem.UndoCommand("DeleteEvent", "temp", incremental_save=False)
 	def addEventFromURL(self, start, url):
 		"""
 		Adds an Event from a URL to this Instrument.
@@ -656,6 +670,9 @@ class Instrument(gobject.GObject):
 			self.events.remove(ev)
 			raise UndoSystem.CancelUndoCommand()
 		
+		inc = IncrementalSave.StartDownload(self.id, url, newfile, start, event_id)
+		self.project.SaveIncrementalAction(inc)
+		
 		self.temp = ev.id
 		self.emit("event::added", ev)
 		
@@ -687,6 +704,7 @@ class Instrument(gobject.GObject):
 		
 		self.temp = ev.id
 		self.emit("event::added", ev)
+		return ev
 	
 	#_____________________________________________________________________
 	
@@ -771,11 +789,21 @@ class Instrument(gobject.GObject):
 		if self.volume != volume:
 			self.volume = volume
 			self.UpdateVolume()
-			self.project.unsavedChanges = True
 			self.emit("volume")
 
 	#_____________________________________________________________________
 	
+	def CommitVolume(self):
+		"""
+		Signal that the volume is no longer volatile. This means we can incrementally save,
+		which we didn't do before because the volume was rapidly changing as the user
+		moved the mouse.
+		"""
+		inc = IncrementalSave.InstrumentSetVolume(self.id, self.volume)
+		self.project.SaveIncrementalAction(inc)
+		
+	#_____________________________________________________________________
+	
 	def UpdateVolume(self):
 		"""
 		Updates the volume property of the gstreamer volume element
@@ -895,6 +923,16 @@ class Instrument(gobject.GObject):
 	
 	#_____________________________________________________________________
 	
+	def SetInput(self, device, inTrack):
+		if device != self.input or inTrack != self.inTrack:
+			self.input = device
+			self.inTrack = inTrack
+			
+			inc = IncrementalSave.InstrumentSetInput(self.id, device, inTrack)
+			self.project.SaveIncrementalAction(inc)
+			
+	#_____________________________________________________________________
+	
 	def OnMute(self):
 		"""
 		Updates the GStreamer volume element to reflect the mute status.
@@ -987,10 +1025,8 @@ class Instrument(gobject.GObject):
 		self.temp = self.instrType
 		self.temp2 = self.name
 		
-		pixbufList = [x[2] for x in Globals.getCachedInstruments() if x[1] == type]
-		if type != self.instrType and pixbufList:
-			pixbuf = pixbufList[0]
-		else:
+		pixbuf = Globals.getCachedInstrumentPixbuf(type)
+		if type == self.instrType or not pixbuf:
 			raise UndoSystem.CancelUndoCommand()
 
 		for tuple_ in Globals.getCachedInstruments():
