@@ -42,6 +42,14 @@ class InstrumentViewer(gtk.EventBox):
 					   gtk.TARGET_SAME_APP,		# Only move inside Jo
 					   INSTR_DRAG_TYPE )]		# Use the custom number
 	
+	"""
+	   The events we wish to receive after we grab the mouse (and it is no longer above this widget)
+	   If events other than mouse event are put in here, it may cause the program to crash.
+	"""
+	_POINTER_GRAB_EVENTS = (
+			gtk.gdk.BUTTON_RELEASE_MASK |
+			gtk.gdk.BUTTON_PRESS_MASK)
+	
 	#_____________________________________________________________________
 	
 	def __init__(self, project, instrument, projectview, mainview, small = False):
@@ -76,11 +84,9 @@ class InstrumentViewer(gtk.EventBox):
 		self.headerBox = gtk.VBox()
 		self.headerEventBox = gtk.EventBox()
 		self.headerEventBox.add(self.headerBox)
-		self.headerAlign = gtk.Alignment(0, 0, 1.0, 1.0)
-		self.headerAlign.add(self.headerEventBox)
 		self.eventLane = EventLaneViewer(project, instrument, self, mainview, self.small)
 		
-		self.mainBox.pack_start(self.headerAlign, False, False)
+		self.mainBox.pack_start(self.headerEventBox, False, False)
 		self.mainBox.pack_end(self.eventLane, True, True)
 
 		# create track header bits
@@ -90,8 +96,14 @@ class InstrumentViewer(gtk.EventBox):
 		self.instrlabel = gtk.Label(self.instrument.name)
 		self.instrlabel.set_ellipsize(pango.ELLIPSIZE_END)
 		self.instrlabel.set_width_chars(self.LABEL_WIDTH_CHARS)
+		
 		self.editlabel = gtk.Entry()
 		self.editlabel.set_text(self.instrument.name)
+		self.editlabel.connect("activate", self.OnAcceptEditLabel)
+		self.editlabel.connect("key_press_event", self.OnEditLabelKey)
+		self.editlabel.connect_after("button-press-event", self.OnEditLabelClick)
+		self.editlabel.connect_after("button-release-event", self.OnEditLabelClick)
+		self.editlabel_is_mouse_down = False
 		self.editlabelPacked = False
 
 		# add the label to the event box
@@ -152,6 +164,14 @@ class InstrumentViewer(gtk.EventBox):
 			self.controlsBox.hide()
 
 	#_____________________________________________________________________
+	
+	def GetHeaderWidget(self):
+		"""
+			Returns the widget which is required to be aligned with the instrument headers.
+		"""
+		return self.headerBox
+	
+	#_____________________________________________________________________
 
 	def OnSelect(self, widget, event):
 		"""
@@ -165,6 +185,7 @@ class InstrumentViewer(gtk.EventBox):
 		Returns:
 			True -- continue GTK signal propagation. *CHECK*
 		"""
+		
 		if self.instrument.project.GetIsRecording():
 			return
 		
@@ -189,35 +210,53 @@ class InstrumentViewer(gtk.EventBox):
 			event -- GTK callback.
 			
 		Returns:
-			True -- continue GTK signal propagation. *CHECK*
+			True -- continue GTK signal propagation.
 		"""
 		if not self.instrument.isSelected:
-			self.OnSelect(widget, event)
 			# Don't edit label unless the user clicks while we are already selected
-			return True
+			# return false so that the event is handled by the parent widget.
+			return False
 			
 		# replace label with gtk.Entry in order to edit it
 		if event.type == gtk.gdk.BUTTON_PRESS:
 			#save width of label because when its hidden, width == 0
 			width = self.labeleventbox.size_request()[0]
-			self.labeleventbox.hide_all()
 			
-			self.editlabel = gtk.Entry()
 			self.editlabel.set_text(self.instrument.name)
 			#set the entry to the same width as the label so that the header doesnt resize
 			self.editlabel.set_size_request(width, -1)
-			self.editlabel.connect("activate", self.OnAcceptEditLabel)
-			self.editlabel.connect("key_press_event", self.OnEditLabelKey)
 			self.editlabel.show()
 			
+			self.labelbox.remove(self.labeleventbox)
 			self.labelbox.pack_end(self.editlabel)
+			
+			self.editlabel.grab_add()
 			self.editlabel.grab_focus()
+			
 			self.editlabelPacked = True
+			self.editlabel_is_mouse_down = False
 			self.mainview.instrNameEntry = self.editlabel
 			
 			return True
 	
 	#_____________________________________________________________________
+	
+	def OnEditLabelClick(self, widget, event):
+		"""
+		Handles the button presses while doing a grab on the editing label.
+		We need to keep track of both BUTTON_PRESS and BUTTON_RELEASE because:
+			1) if the mouse click is on the gtk.Entry, release will be forwarded, press will not.
+			2) if the mouse is outside the gtk.Entry, both press and release will be forwarded.
+		Therefore if we get both press and release together, we accept the changed and remove the edit dialog.
+		"""
+		if event.type == gtk.gdk.BUTTON_PRESS:
+			self.editlabel_is_mouse_down = True
+			return True
+		elif event.type == gtk.gdk.BUTTON_RELEASE and self.editlabel_is_mouse_down:
+			self.OnAcceptEditLabel()
+			return True
+
+	#_____________________________________________________________________
 
 	def OnEditLabelKey(self, widget, event):
 		"""
@@ -233,6 +272,7 @@ class InstrumentViewer(gtk.EventBox):
 		if key == "Escape":
 			self.editlabel.set_text("")
 			self.OnAcceptEditLabel()
+			return True
 
 	#_____________________________________________________________________
 
@@ -244,6 +284,7 @@ class InstrumentViewer(gtk.EventBox):
 		Parameters:
 			widget -- GTK callback.
 		"""
+		self.editlabel.grab_remove()
 		# change instrument name then replace edit label with normal label
 		if self.editlabelPacked:
 			name = self.editlabel.get_text()
@@ -252,9 +293,7 @@ class InstrumentViewer(gtk.EventBox):
 			self.labelbox.remove(self.editlabel)
 			self.editlabelPacked = False
 			self.mainview.instrNameEntry = None
-			if self.editlabel:
-				self.editlabel.destroy()
-			self.labeleventbox.show_all()
+			self.labelbox.pack_end(self.labeleventbox)
 			
 			if name != "" and name != self.instrument.name:
 				#this must be done last because it triggers update
@@ -324,23 +363,6 @@ class InstrumentViewer(gtk.EventBox):
 			
 	#______________________________________________________________________
 
-	def ResizeHeader(self, width):
-		"""
-		Changes the padding space of the header box in order to line
-		up correctly with the timeline display.
-		
-		Considerations:
-			This method should be called from TimeLineBar.
-			
-		Parameters:
-			width -- new size of the header box.
-		"""
-		padding = width - self.headerBox.size_request()[0]
-		self.headerAlign.set_padding(0, 0, 0, padding)
-
-	#______________________________________________________________________
-
-
 	def OnDragMotion(self, widget, context, x, y, time):
 		"""
 		Called each time the user moves the mouse while dragging.
