@@ -12,8 +12,7 @@ import pygtk
 pygtk.require("2.0")
 import gtk.glade, gobject
 import sys
-import os, os.path
-import time
+import os.path
 import pygst
 pygst.require("0.10")
 import gst
@@ -24,11 +23,10 @@ _ = gettext.gettext
 
 import AddInstrumentDialog, TimeView, Workspace
 import PreferencesDialog, ExtensionManagerDialog, RecordingView, NewProjectDialog
-import ProjectManager, Globals, WelcomeDialog, AlsaDevices
+import ProjectManager, Globals, WelcomeDialog
 import InstrumentConnectionsDialog, StatusBar
 import EffectPresets, Extension, ExtensionManager
 import Utils, AudioPreview, MixdownProfileDialog, MixdownActions
-import CrashProtectionDialog
 
 #=========================================================================
 
@@ -115,7 +113,7 @@ class MainApp:
 			"on_report_bug_activate" : self.OnReportBug,
 			"on_project_add_audio" : self.OnAddAudioFile,
 			"on_system_information_activate" : self.OnSystemInformation,
-			"on_restore_crashed_project_activate" : self.OnRestoreCrashedProject,
+			"on_properties_activate" : self.OnProjectProperties,
 		}
 		self.wTree.signal_autoconnect(signals)
 		
@@ -139,7 +137,6 @@ class MainApp:
 		self.copy = self.wTree.get_widget("copy")
 		self.paste = self.wTree.get_widget("paste")
 		self.delete = self.wTree.get_widget("delete")
-		self.projectMenu = self.wTree.get_widget("projectmenu")
 		self.instrumentMenu = self.wTree.get_widget("instrumentmenu")
 		self.export = self.wTree.get_widget("export")
 		self.recentprojects = self.wTree.get_widget("recentprojects")
@@ -151,6 +148,10 @@ class MainApp:
 		self.removeInstrMenuItem = self.wTree.get_widget("remove_selected_instrument")
 		self.addAudioFileButton = self.wTree.get_widget("addAudioFileButton")
 		self.addAudioFileMenuItem = self.wTree.get_widget("add_audio_file_project_menu")
+		self.addInstrumentFileMenuItem = self.wTree.get_widget("add_instrument1")
+		self.recordingInputsFileMenuItem = self.wTree.get_widget("instrument_connections1")
+		self.timeFormatFileMenuItem = self.wTree.get_widget("time_format1")
+		self.properties_menu_item = self.wTree.get_widget("project_properties")
 		
 		self.recentprojectitems = []
 		self.lastopenedproject = None
@@ -201,11 +202,7 @@ class MainApp:
 		miximg.show()
 		
 		#get the audiofile image from Globals
-		self.audioFilePixbuf = None
-		for name, type, pixbuf, path in Globals.getCachedInstruments():
-			if type == "audiofile":
-				self.audioFilePixbuf = pixbuf
-				break
+		self.audioFilePixbuf = Globals.getCachedInstrumentPixbuf("audiofile")
 		
 		audioimg = gtk.Image()
 		size = gtk.icon_size_lookup(gtk.ICON_SIZE_MENU)
@@ -247,6 +244,7 @@ class MainApp:
 		# set up presets registry - this should probably be removed here	
 		EffectPresets.EffectPresets()
 		Globals.PopulateEncoders()
+		Globals.PopulateAudioBackends()
 		
 		# seems like this is the best place to instantiate RegisterMixdownActionAPI
 		# as extensions and the mixdown profile dialog can use it through mainapp
@@ -265,28 +263,6 @@ class MainApp:
 		# Show the main window
 		self.window.show_all()
 
-		self.backupProject = None
-		self.restoredProject = False
-
-		# Check for crash and offer recovery
-		backupDir = os.path.join(os.path.expanduser("~"), ".jokosher", "backups")
-		if not os.path.exists(backupDir):
-			os.mkdir(backupDir)
-
-		for backupFile in os.listdir(backupDir):
-			backup = os.path.join(backupDir, backupFile)
-			if os.stat(backup).st_size > 0 and os.stat(backup).st_mtime > float(Globals.settings.general["lastbackup"]):
-				# We didn't shutdown cleanly last time, offer recovery
-				CrashProtectionDialog.CrashProtectionDialog(self, True)
-				break
-
-		# Backup saving
-		self.SetupBackup()
-
-		if self.restoredProject:
-			#Don't display the welcome dialog if we've just restored a project (since it'll have been opened)
-			return
-
 		# command line options override preferences so check for them first,
 		# then preferences, then default to the welcome dialog
 		if startuptype == 2: # welcomedialog cmdline switch
@@ -317,7 +293,17 @@ class MainApp:
 		"""
 		if self.workspace:
 			self.workspace.ToggleCompactMix()
-				
+
+	#_____________________________________________________________________
+
+
+	def OnF3Pressed(self):
+		"""
+		Toggle to compact mix view button when F3 is pressed.
+		"""
+
+		self.compactMixButton.set_active(not self.compactMixButton.get_active())
+
 	#_____________________________________________________________________
 	
 	def OnResize(self, widget, event):
@@ -449,29 +435,25 @@ class MainApp:
 				canRecord = True
 
 		#Check to see if any instruments are trying to use the same input channel
-		usedChannels = {}
-		for instr in self.project.instruments:
-			if instr.isArmed:
-				if usedChannels.has_key(instr.input):
-					if usedChannels[instr.input].has_key(instr.inTrack):
-						string = _("The instruments '%(name1)s' and '%(name2)s' both have the same input selected (%(track)s). Please either disarm one, or connect it to a different input through 'Project -> Recording Inputs'")
-						message = string % {"name1":usedChannels[instr.input][instr.inTrack], "name2":instr.name, "track":instr.inTrack}
-						dlg = gtk.MessageDialog(self.window,
-							gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
-							gtk.MESSAGE_INFO,
-							gtk.BUTTONS_CLOSE,
-							message)
-						dlg.connect('response', lambda dlg, response: dlg.destroy())
-						dlg.run()
-						self.settingButtons = True
-						widget.set_active(False)
-						self.settingButtons = False
-						return
-					else:
-						usedChannels[instr.input][instr.inTrack] = instr.name
-				else:
-					usedChannels[instr.input] = {instr.inTrack : instr.name}
-				
+		usedChannels = []
+		armed_instrs = [x for x in self.project.instruments if x.isArmed]
+		for instrA in armed_instrs:
+			for instrB in armed_instrs:
+				if instrA is not instrB and instrA.input == instrB.input and instrA.inTrack == instrB.inTrack:
+					string = _("The instruments '%(name1)s' and '%(name2)s' both have the same input selected. Please either disarm one, or connect it to a different input through 'Project -> Recording Inputs'")
+					message = string % {"name1" : instrA.name, "name2" : instrB.name}
+					dlg = gtk.MessageDialog(self.window,
+						gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+						gtk.MESSAGE_INFO,
+						gtk.BUTTONS_CLOSE,
+						message)
+					dlg.connect('response', lambda dlg, response: dlg.destroy())
+					dlg.run()
+					self.settingButtons = True
+					widget.set_active(False)
+					self.settingButtons = False
+					return
+
 		if not canRecord:
 			Globals.debug("can not record")
 			if self.project.instruments:
@@ -490,25 +472,7 @@ class MainApp:
 			self.settingButtons = False
 		else:
 			Globals.debug("can record")
-			
-			try:
-				self.project.Record()
-			except ProjectManager.AudioInputsError, e:
-				if e.errno==0:
-					message=_("No channels capable of recording have been found, please attach a device and try again.")
-				elif e.errno==1:
-					message=_("Your sound card isn't capable of recording from multiple sources at the same time. Please disarm all but one instrument.")
-				elif e.errno==2:
-					message=_("You require the GStreamer channel splitting element to be able to record from multiple input devices. This can be downloaded from http://www.jokosher.org/download.")
-
-				dlg = gtk.MessageDialog(self.window,
-					gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
-					gtk.MESSAGE_INFO,
-					gtk.BUTTONS_CLOSE,
-					message)
-				dlg.connect('response', lambda dlg, response: dlg.destroy())
-				dlg.run()
-				self.project.TerminateRecording()
+			self.project.Record()
 
 	#_____________________________________________________________________
 	
@@ -802,11 +766,6 @@ class MainApp:
 			self.project.SelectInstrument(None)
 			self.project.ClearEventSelections()
 			self.project.SaveProjectFile()
-			#Remove backup
-			if os.path.exists(self.backupProject):
-				Globals.debug("Removing backup file.")
-				os.remove(self.backupProject)
-
 			
 	#_____________________________________________________________________
 	
@@ -841,18 +800,6 @@ class MainApp:
 		
 	#_____________________________________________________________________
 
-	def BackupSave(self):
-		"""
-		Saves the project file to a backup in case of crashes.
-		"""
-		if self.project:
-			Globals.debug("Making automatic backup")
-			self.project.SaveProjectFile(self.backupProject, True)
-
-		gobject.timeout_add(int(Globals.settings.general["backupsavetime"]), self.BackupSave)
-
-	#_____________________________________________________________________
-
 	def OnNewProject(self, widget, destroyCallback=None):
 		"""
 		Creates and shows the "New Project" dialog.
@@ -921,11 +868,6 @@ class MainApp:
 		
 		self.project.CloseProject()
 
-		#Remove backup
-		if os.path.exists(self.backupProject):
-			Globals.debug("Removing backup file.")
-			os.remove(self.backupProject)
-
 		self.project = None
 		self.mode = None
 		return 0
@@ -968,7 +910,7 @@ class MainApp:
 		self.stop.set_sensitive(True)	#stop should always be clickable
 		self.record.set_sensitive(not self.isPlaying)
 		
-		controls = (self.play, self.reverse, self.forward, self.editmenu, self.projectMenu, self.instrumentMenu, 
+		controls = (self.play, self.reverse, self.forward, self.editmenu, self.instrumentMenu, 
 				self.workspace.recordingView.timelinebar.headerhbox, 
 				self.addInstrumentButton, self.addAudioFileButton)
 		for widget in controls:
@@ -1027,31 +969,6 @@ class MainApp:
 	
 	#_____________________________________________________________________
 	
-	def OnProjectGstError(self, project, error, details):
-		"""
-		Callback for when the project sends a gstreamer error message
-		from the pipeline.
-		
-		Parameters:
-			project -- The project instance that send the signal.
-			error -- The type of error that occurred as a string.
-			details -- A string with more information about the error.
-		"""
-		introstring = _("Argh! Something went wrong and a serious error occurred:")
-		outrostring = _("It is recommended that you report this to the Jokosher developers or get help at http://www.jokosher.org/forums/")
-		
-		outputtext = "\n\n".join((introstring, error, details, outrostring))
-		
-		dlg = gtk.MessageDialog(self.window,
-			gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
-			gtk.MESSAGE_ERROR,
-			gtk.BUTTONS_CLOSE,
-			outputtext)
-		dlg.connect('response', lambda dlg, response: dlg.destroy())
-		dlg.show()
-		
-	#_____________________________________________________________________
-	
 	def OnProjectUndo(self, project=None):
 		"""
 		Callback for when the project's undo or redo stacks change.
@@ -1345,8 +1262,10 @@ class MainApp:
 		
 		ctrls = (self.save, self.save_as, self.close, self.addInstrumentButton, self.addAudioFileButton,
 			self.reverse, self.forward, self.play, self.stop, self.record,
-			self.projectMenu, self.instrumentMenu, self.export, self.cut, self.copy, self.paste,
-			self.undo, self.redo, self.delete, self.compactMixButton)
+			self.instrumentMenu, self.export, self.cut, self.copy, self.paste,
+			self.undo, self.redo, self.delete, self.compactMixButton, self.properties_menu_item,
+			self.addAudioFileMenuItem, self.addInstrumentFileMenuItem, self.recordingInputsFileMenuItem,
+			self.timeFormatFileMenuItem)
 		
 		if self.project:
 			# make various buttons and menu items enabled now we have a project option
@@ -1416,7 +1335,7 @@ class MainApp:
 		else:
 			keysdict = {
 				"F1"			: self.OnHelpContentsMenu, # F1 - Help Contents
-				"F3"			: self.OnCompactMixView, # F3 - Compact Mix View
+				"F3"			: self.OnF3Pressed, # F3 - Compact Mix View
 				"Delete"		: self.OnDelete, # delete key - remove selected item
 				"BackSpace" 	: self.OnDelete, # backspace key
 				"space"		: self.Play,
@@ -1456,12 +1375,18 @@ class MainApp:
 		self.PopulateMixdownAsMenu()
 		if self.isRecording:
 			self.export.set_sensitive(False)
+			self.addInstrumentFileMenuItem.set_sensitive(False)
+			self.addAudioFileMenuItem.set_sensitive(False)
+			self.recordingInputsFileMenuItem.set_sensitive(False)
 			if self.mixdown_as_header:
 				self.mixdown_as_header.set_sensitive(False)
 			return
 		
 		eventList = False
 		if self.project:
+			self.addInstrumentFileMenuItem.set_sensitive(True)
+			self.addAudioFileMenuItem.set_sensitive(True)
+			self.recordingInputsFileMenuItem.set_sensitive(True)
 			for instr in self.project.instruments:
 				if instr.events:
 					eventList = True
@@ -1581,7 +1506,6 @@ class MainApp:
 			
 		self.project = project
 		
-		self.project.connect("gst-bus-error", self.OnProjectGstError)
 		self.project.connect("audio-state::play", self.OnProjectAudioState)
 		self.project.connect("audio-state::pause", self.OnProjectAudioState)
 		self.project.connect("audio-state::record", self.OnProjectAudioState)
@@ -1809,20 +1733,6 @@ class MainApp:
 	
 	#_____________________________________________________________________
 
-	def OnRestoreCrashedProject(self, widget):
-		"""
-		Displays a dialog allowing the user to either select a previously crashed 
-		project for restoration or to delete a crash file.
-
-		Parameters:
-			widget -- GTK callback parameter.
-		"""
-
-		self.crashDialog = CrashProtectionDialog.CrashProtectionDialog(self)
-
-
-	#_____________________________________________________________________
-
 	def ShowOpenProjectErrorDialog(self, error, parent=None):
 		"""
 		Creates and shows a dialog to inform the user about an error that has ocurred.
@@ -1988,7 +1898,7 @@ class MainApp:
 		the file menu and add links to them.
 		"""
 		self.mixdown_as_header = None
-		savefolder = os.path.expanduser('~/.jokosher/mixdownprofiles') # created by Globals
+		savefolder = os.path.join(Globals.JOKOSHER_DATA_HOME, 'mixdownprofiles') # created by Globals
 		profiles = os.listdir(savefolder)
 		if not profiles: return
 		
@@ -2004,12 +1914,12 @@ class MainApp:
 		# If there's already a Mixdown As submenu, delete it and recreate it
 		for i in filemenulist.get_children():
 			if i.get_children():
-				if i.get_children()[0].get_label() == _("Mixdown as"):
+				if i.get_children()[0].get_label() == _("Mix_down As"):
 					filemenulist.remove(i)
 					i.destroy()
 		
 		# Create a Mixdown As submenu header
-		self.mixdown_as_header = gtk.MenuItem(label=_("Mixdown as"))
+		self.mixdown_as_header = gtk.MenuItem(label=_("Mix_down As"))
 		submenu = gtk.Menu()
 		for p in profiles:
 			profilenames = p.split(".")[0]
@@ -2031,32 +1941,55 @@ class MainApp:
 		self.filemenu.show_all()
 		
 	#_____________________________________________________________________
-
-	def SetupBackup(self, num=0):
-		"""
-		Sets up the backup system for crash protection. Stores all backups in 
-		~/.jokosher/backups in the format timestamp-num.jokosher. Backups will
-		occur at an interval specified in the "backupsavetime" config option.
-
-		Parameters:
-			num -- Number appended to the timestamp to allow for multiple
-			copies of Jokosher being launched at the same time.
-		"""
-
-		backupDir = os.path.join(os.path.expanduser("~"), ".jokosher", "backups")
-		backupFile = "%d-%d.jokosher" % (int(time.time()), num)
-		self.backupProject = os.path.join(backupDir, backupFile)
-		if os.path.exists(self.backupProject):
-			#Multiple copies of Jokosher have been opened simultaneously, so increment
-			#num to avoid conflicts
-			self.SetupBackup(num+1)
-		else:
-			#Open the file quickly so other instances can see it (still potential for 
-			#a race condition, but chances are greatly reduced)
-			open(self.backupProject, "w")
-			gobject.timeout_add(int(Globals.settings.general["backupsavetime"]), self.BackupSave)
-
 	
+	def OnProjectProperties(self, widget=None):
+		"""
+		Called when the "Properties..." in the project menu is clicked.
+		
+		Parameters:
+			widget -- reserved for GTK callbacks, don't use it explicitly.
+		"""
+		if not self.project:
+			return
+		
+		propertiesTree = gtk.glade.XML(Globals.GLADE_PATH, "ProjectPropertiesDialog")
+		dlg = propertiesTree.get_widget("ProjectPropertiesDialog")
+		nameEntry = propertiesTree.get_widget("nameEntry")
+		authorEntry = propertiesTree.get_widget("authorEntry")
+		notesTextView = propertiesTree.get_widget("notesTextView")
+		
+		nameEntry.set_text(self.project.name)
+		authorEntry.set_text(self.project.author)
+		buffer = gtk.TextBuffer()
+		buffer.set_text(self.project.notes)
+		notesTextView.set_buffer(buffer)
+		
+		dlg.connect("response", self.OnProjectPropertiesClose, nameEntry, authorEntry, notesTextView)
+		dlg.show_all()
+		
+	#_____________________________________________________________________
+		
+	def OnProjectPropertiesClose(self, dialog, response, nameEntry, authorEntry, notesTextView):
+		"""
+		Called when the "Project Properties" windows is closed.
+		
+		Parameters:
+			dialog -- reserved for GTK callbacks, don't use it explicitly.
+		"""
+		
+		if self.project and response == gtk.RESPONSE_CLOSE:
+			name = nameEntry.get_text()
+			author = authorEntry.get_text()
+			buffer = notesTextView.get_buffer()
+			notes = buffer.get_text(*buffer.get_bounds())
+			
+			self.project.SetName(name)
+			self.project.SetAuthor(author)
+			self.project.SetNotes(notes)
+				
+		dialog.destroy()
+
+	#_____________________________________________________________________
 #=========================================================================
 
 def main():
