@@ -20,12 +20,12 @@ import gzip
 import re
 
 import TransportManager
-import UndoSystem
+import UndoSystem, IncrementalSave
 import Globals
 import xml.dom.minidom as xml
 import Instrument, Event
 import Utils
-import AlsaDevices
+import AudioBackend
 import ProjectManager
 
 #=========================================================================
@@ -37,11 +37,15 @@ class Project(gobject.GObject):
 	"""
 	
 	""" The Project structure version. Will be useful for handling old save files. """
-	Globals.VERSION = "0.10"
+	Globals.VERSION = "0.11.1"
 	
 	""" The audio playback state enum values """
 	AUDIO_STOPPED, AUDIO_RECORDING, AUDIO_PLAYING, AUDIO_PAUSED, AUDIO_EXPORTING = range(5)
 	
+	""" String constants for incremental save """
+	INCREMENTAL_SAVE_EXT = ".incremental"
+	INCREMENTAL_SAVE_DELIMITER = "\n<<delimiter>>\n"
+	
 	"""
 	Signals:
 		"audio-state" -- The status of the audio system has changed. See below:
@@ -52,8 +56,9 @@ class Project(gobject.GObject):
 			"audio-state::export-start" -- The audio is being played to a file.
 			"audio-state::export-stop" -- The export to a file has completed.
 		"bpm" -- The beats per minute value was changed.
-		"click-track" -- The click track was turned on or off.
+		"click-track" -- The volume of the click track changed.
 		"gst-bus-error" -- An error message was posted to the pipeline. Two strings are also send with the error details.
+		"incremental-save" -- An action was logged to the .incremental file.
 		"instrument" -- The instruments for this project have changed. The instrument instance will be passed as a parameter. See below:
 			"instrument::added" -- An instrument was added to this project.
 			"instrument::removed" -- An instrument was removed from this project.
@@ -68,8 +73,9 @@ class Project(gobject.GObject):
 	__gsignals__ = {
 		"audio-state"		: ( gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_DETAILED, gobject.TYPE_NONE, () ),
 		"bpm"			: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
-		"click-track"		: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
+		"click-track"		: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_DOUBLE,) ),
 		"gst-bus-error"	: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING, gobject.TYPE_STRING) ),
+		"incremental-save" : ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
 		"instrument"		: ( gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_DETAILED, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,) ),
 		"time-signature"	: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
 		"undo"			: ( gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, () ),
@@ -86,8 +92,9 @@ class Project(gobject.GObject):
 		"""
 		gobject.GObject.__init__(self)
 		
-		self.author = ""			#the author of this project
+		self.author = ""			#user specified author of this project
 		self.name = ""				#the name of this project
+		self.notes = ""				#user specified notes for the project
 		self.projectfile = ""		#the name of the project file, complete with path
 		self.audio_path = ""
 		self.levels_path = ""
@@ -107,15 +114,18 @@ class Project(gobject.GObject):
 		self.meter_nom = 4		# time signature numerator
 		self.meter_denom = 4		# time signature denominator
 		self.clickbpm = 120			#the number of beats per minute that the click track will play
-		self.clickEnabled = False	#True is the click track is currently enabled
+		self.clickVolumeValue = 0	#The value of the click track volume between 0.0 and 1.0
 		#Keys are instruments which are recording; values are 3-tuples of the event being recorded, the recording bin and bus handler id
 		self.recordingEvents = {}	#Dict containing recording information for each recording instrument
 		self.volume = 1.0			#The volume setting for the entire project
 		self.level = 0.0			#The level of the entire project as reported by the gstreamer element
 		self.currentSinkString = None	#to keep track if the sink changes or not
 
+		self.hasDoneIncrementalSave = False	# True if we have already written to the .incremental file from this project.
+		self.isDoingIncrementalRestore = False # If we are currently restoring incremental save actions
+
 		# Variables for the undo/redo command system
-		self.unsavedChanges = False		#This boolean is to indicate if something which is not on the undo/redo stack needs to be saved
+		self.__unsavedChanges = False	#This boolean is to indicate if something which is not on the undo/redo stack needs to be saved
 		self.__undoStack = []			#not yet saved undo commands
 		self.__redoStack = []			#not yet saved actions that we're undone
 		self.__savedUndoStack = []		#undo commands that have already been saved in the project file
@@ -129,27 +139,30 @@ class Project(gobject.GObject):
 		self.mainpipeline = gst.Pipeline("timeline")
 		self.playbackbin = gst.Bin("playbackbin")
 		self.adder = gst.element_factory_make("adder")
+		self.postAdderConvert = gst.element_factory_make("audioconvert")
 		self.masterSink = self.MakeProjectSink()
 		
 		self.levelElement = gst.element_factory_make("level", "MasterLevel")
 		self.levelElement.set_property("interval", gst.SECOND / 50)
 		self.levelElement.set_property("message", True)
 		
-		#Restrict adder's output caps due to adder bug
+		#Restrict adder's output caps due to adder bug 341431
 		self.levelElementCaps = gst.element_factory_make("capsfilter", "levelcaps")
-		caps = gst.caps_from_string("audio/x-raw-int,rate=44100,channels=2,width=16,depth=16,signed=(boolean)true")
+		capsString = "audio/x-raw-float,rate=44100,channels=2,width=32,endianness=1234"
+		caps = gst.caps_from_string(capsString)
 		self.levelElementCaps.set_property("caps", caps)
 		
 		# ADD ELEMENTS TO THE PIPELINE AND/OR THEIR BINS #
 		self.mainpipeline.add(self.playbackbin)
 		Globals.debug("added project playback bin to the pipeline")
-		for element in [self.adder, self.levelElementCaps, self.levelElement, self.masterSink]:
+		for element in [self.adder, self.levelElementCaps, self.postAdderConvert, self.levelElement, self.masterSink]:
 			self.playbackbin.add(element)
 			Globals.debug("added %s to project playbackbin" % element.get_name())
 
 		# LINK GSTREAMER ELEMENTS #
 		self.adder.link(self.levelElementCaps)
-		self.levelElementCaps.link(self.levelElement)
+		self.levelElementCaps.link(self.postAdderConvert)
+		self.postAdderConvert.link(self.levelElement)
 		self.levelElement.link(self.masterSink)
 		
 		# CONSTRUCT CLICK TRACK BIN #
@@ -243,7 +256,7 @@ class Project(gobject.GObject):
 		Globals.debug("current state:", self.mainpipeline.get_state(0)[1].value_name)
 		
 		#If we've been recording then add new events to instruments
-		for instr, (event, bin, handle) in self.recordingEvents.items():
+		for instr, (event, bin, handle) in self.recordingEvents.iteritems():
 			instr.FinalizeRecording(event)
 			self.bus.disconnect(handle)
 
@@ -263,7 +276,7 @@ class Project(gobject.GObject):
 		Globals.debug("State just set to READY")
 		
 		#Relink instruments and stop their recording bins
-		for instr, (event, bin, handle) in self.recordingEvents.items():
+		for instr, (event, bin, handle) in self.recordingEvents.iteritems():
 			try:
 				Globals.debug("Removing recordingEvents bin")
 				self.mainpipeline.remove(bin)
@@ -287,50 +300,75 @@ class Project(gobject.GObject):
 		#Add all instruments to the pipeline
 		self.recordingEvents = {}
 		devices = {}
-		for device in AlsaDevices.GetAlsaList("capture").keys():
+		capture_devices = AudioBackend.ListCaptureDevices(probe_name=False)
+		if not capture_devices:
+			capture_devices = ((None,None),)
+		
+		default_device = capture_devices[0][0]
+		
+		for device, deviceName in capture_devices:
 			devices[device] = []
 			for instr in self.instruments:
-				if instr.isArmed and instr.input == device:
+				if instr.isArmed and (instr.input == device or device is None):
 					instr.RemoveAndUnlinkPlaybackbin()
 					devices[device].append(instr)
+				elif instr.isArmed and instr.input is None:
+					instr.RemoveAndUnlinkPlaybackbin()
+					devices[default_device].append(instr)
+		
 
-		for device, recInstruments in devices.items():
+		for device, recInstruments in devices.iteritems():
 			if len(recInstruments) == 0:
 				#Nothing to record on this device
 				continue
 
-			channelsNeeded = AlsaDevices.GetChannelsOffered(device)
-
-			if channelsNeeded > 1 and not gst.registry_get_default().find_plugin("chansplit"):
-				Globals.debug("Channel splitting element not found when trying to record from multi-input device.")
-				raise ProjectManager.AudioInputsError(2)
+			if device is None:
+				# assume we are using a backend like JACK which does not allow
+				#us to do device selection.
+				channelsNeeded = len(recInstruments)
+			else:
+				channelsNeeded = AudioBackend.GetChannelsOffered(device)
 
+			
 			if channelsNeeded > 1: #We're recording from a multi-input device
-				recordingbin = gst.Bin()
-				src = gst.element_factory_make("alsasrc")
-				src.set_property("device", device)
-				
-				capsfilter = None
-				sampleRate = Globals.settings.recording["samplerate"]
-				# 0 means for "autodetect", or more technically "don't use any caps".
-				if sampleRate > 0:
-					gst.element_factory_make("capsfilter")
-					capsString = "audio/x-raw-int,rate=%s" % sampleRate
-					caps = gst.caps_from_string(capsString)
-					capsfilter.set_property("caps", caps)
-				
-				split = gst.element_factory_make("chansplit")
-				
-				recordingbin.add(src)
-				if capsfilter:
-					recordingbin.add(capsfilter)
-				recordingbin.add(split)
-				
-				if capsfilter:
-					src.link(capsfilter)
-					capsfilter.link(split)
+				recordingbin = gst.Bin("recording bin")
+				recordString = Globals.settings.recording["audiosrc"]
+				srcBin = gst.parse_bin_from_description(recordString, True)
+				try:
+					src_element = recordingbin.iterate_sources().next()
+				except StopIteration:
+					pass
 				else:
-					src.link(split)
+					if hasattr(src_element.props, "device"):
+						src_element.set_property("device", device)
+				
+				caps = gst.caps_from_string("audio/x-raw-int;audio/x-raw-float")
+
+				sampleRate = Globals.settings.recording["samplerate"]
+				try:
+					sampleRate = int(sampleRate)
+				except ValueError:
+					sampleRate = 0
+				# 0 means for "autodetect", or more technically "don't use any rate caps".
+				if sampleRate > 0:
+					for struct in caps:
+						struct.set_value("rate", sampleRate)
+
+				for struct in caps:
+					struct.set_value("channels", channelsNeeded)
+
+				Globals.debug("recording with capsfilter:", caps.to_string())
+				capsfilter = gst.element_factory_make("capsfilter")
+				capsfilter.set_property("caps", caps)
+				
+				split = gst.element_factory_make("deinterleave")
+				convert = gst.element_factory_make("audioconvert")
+				
+				recordingbin.add(srcBin, split, convert, capsfilter)
+				
+				srcBin.link(convert) 
+				convert.link(capsfilter)
+				capsfilter.link(split)
 				
 				split.connect("pad-added", self.__RecordingPadAddedCb, recInstruments, recordingbin)
 				Globals.debug("Recording in multi-input mode")
@@ -341,6 +379,7 @@ class Project(gobject.GObject):
 				event = instr.GetRecordingEvent()
 				
 				encodeString = Globals.settings.recording["fileformat"]
+				recordString = Globals.settings.recording["audiosrc"]
 				
 				sampleRate = 0
 				try:
@@ -353,16 +392,24 @@ class Project(gobject.GObject):
 				else:
 					capsString = "audioconvert"
 					
-				pipe = "alsasrc device=%s ! %s ! level name=recordlevel interval=%d" +\
+				pipe = "%s ! %s ! level name=recordlevel interval=%d" +\
 							" ! audioconvert ! %s ! filesink location=%s"
-				pipe %= (device, capsString, event.LEVEL_INTERVAL * gst.SECOND, encodeString, event.file.replace(" ", "\ "))
+				pipe %= (recordString, capsString, event.LEVEL_INTERVAL * gst.SECOND, encodeString, event.file.replace(" ", "\ "))
 				
 				Globals.debug("Using pipeline: %s" % pipe)
 				
-				recordingbin = gst.parse_launch("bin.( %s )" % pipe)
+				recordingbin = gst.parse_bin_from_description(pipe, False)
 				#update the levels in real time
 				handle = self.bus.connect("message::element", event.recording_bus_level)
 				
+				try:
+					src_element = recordingbin.iterate_sources().next()
+				except StopIteration:
+					pass
+				else:
+					if hasattr(src_element.props, "device"):
+						src_element.set_property("device", device)
+				
 				self.recordingEvents[instr] = (event, recordingbin, handle)
 				
 				Globals.debug("Recording in single-input mode")
@@ -389,7 +436,7 @@ class Project(gobject.GObject):
 		"""
 		#try to create encoder/muxer first, before modifying the main pipeline.
 		try:
-			self.encodebin = gst.parse_bin_from_description("audioconvert ! %s" % encodeBin, True)
+			self.encodebin = gst.parse_bin_from_description(encodeBin, True)
 		except gobject.GError, e:
 			if e.code == gst.PARSE_ERROR_NO_SUCH_ELEMENT:
 				error_no = ProjectManager.ProjectExportException.MISSING_ELEMENT
@@ -402,7 +449,7 @@ class Project(gobject.GObject):
 		
 		#remove and unlink the alsasink
 		self.playbackbin.remove(self.masterSink, self.levelElement)
-		self.levelElementCaps.unlink(self.levelElement)
+		self.postAdderConvert.unlink(self.levelElement)
 		self.levelElement.unlink(self.masterSink)
 		
 		#create filesink
@@ -411,7 +458,7 @@ class Project(gobject.GObject):
 		self.playbackbin.add(self.outfile)
 
 		self.playbackbin.add(self.encodebin)
-		self.levelElementCaps.link(self.encodebin)
+		self.postAdderConvert.link(self.encodebin)
 		self.encodebin.link(self.outfile)
 			
 		#disconnect the bus message handler so the levels don't change
@@ -449,7 +496,7 @@ class Project(gobject.GObject):
 		
 		#remove the filesink and encoder
 		self.playbackbin.remove(self.outfile, self.encodebin)		
-		self.levelElementCaps.unlink(self.encodebin)
+		self.postAdderConvert.unlink(self.encodebin)
 			
 		#dispose of the elements
 		self.outfile.set_state(gst.STATE_NULL)
@@ -458,7 +505,7 @@ class Project(gobject.GObject):
 		
 		#re-add all the alsa playback elements
 		self.playbackbin.add(self.masterSink, self.levelElement)
-		self.levelElementCaps.link(self.levelElement)
+		self.postAdderConvert.link(self.levelElement)
 		self.levelElement.link(self.masterSink)
 		
 		self.emit("audio-state::export-stop")
@@ -546,18 +593,22 @@ class Project(gobject.GObject):
 			recInstruments -- list with all Instruments currently recording.
 			bin -- the bin that stores all the recording elements.
 		"""
-		match = re.search("(\d+)$", pad.get_name())
-		if not match:
+		# SRC template: 'src%d'
+		padname = pad.get_name()
+		try:
+			index = int(padname[3:])
+		except ValueError:
+			Globals.debug("Cannot start multichannel record: pad name does not match 'src%d':", padname)
 			return
-		index = int(match.groups()[0])
+
 		for instr in recInstruments:
 			if instr.inTrack == index:
 				event = instr.GetRecordingEvent()
 				
 				encodeString = Globals.settings.recording["fileformat"]
-				pipe = "audioconvert ! level name=eventlevel interval=%d message=true !" +\
+				pipe = "queue ! audioconvert ! level name=recordlevel interval=%d !" +\
 							"audioconvert ! %s ! filesink location=%s"
-				pipe %= (event.LEVEL_INTERVAL, encodeString, event.file.replace(" ", "\ "))
+				pipe %= (event.LEVEL_INTERVAL * gst.SECOND, encodeString, event.file.replace(" ", "\ "))
 				
 				encodeBin = gst.parse_bin_from_description(pipe, True)
 				bin.add(encodeBin)
@@ -565,7 +616,12 @@ class Project(gobject.GObject):
 				
 				handle = self.bus.connect("message::element", event.recording_bus_level)
 				
+				# since we are adding the encodebin to an already playing pipeline, sync up there states
+				encodeBin.set_state(gst.STATE_PLAYING)
+
 				self.recordingEvents[instr] = (event, bin, handle)
+				Globals.debug("Linked recording channel: instrument (%s), track %d" % (instr.name, instr.inTrack))
+				break
 
 	#_____________________________________________________________________
 	
@@ -632,16 +688,26 @@ class Project(gobject.GObject):
 		"""
 		error, debug = message.parse_error()
 		
-		#FIXME: remove these prints!
-		print "*Project Error*"
-		print ("Code: %s, Domain: %s")%(error.code, error.domain)
-		print "Message: "+error.message
-		
 		Globals.debug("Gstreamer bus error:", str(error), str(debug))
+		Globals.debug("Domain: %s, Code: %s" % (error.domain, error.code))
+		Globals.debug("Message:", error.message)
+		
+		if error.domain == gst.STREAM_ERROR and Globals.DEBUG_GST:
+			self.DumpDotFile()
+		
 		self.emit("gst-bus-error", str(error), str(debug))
 
 	#_____________________________________________________________________
 	
+	def DumpDotFile(self):
+		basepath, ext = os.path.splitext(self.projectfile)
+		name = "jokosher-pipeline-" + os.path.basename(basepath)
+		gst.DEBUG_BIN_TO_DOT_FILE_WITH_TS(self.mainpipeline, gst.DEBUG_GRAPH_SHOW_ALL, name)
+		Globals.debug("Dumped pipeline to DOT file:", name)
+		Globals.debug("Command to render DOT file: dot -Tsvg -o pipeline.svg <file>")
+	
+	#_____________________________________________________________________
+	
 	def SaveProjectFile(self, path=None, backup=False):
 		"""
 		Saves the Project and its children as an XML file
@@ -653,7 +719,7 @@ class Project(gobject.GObject):
 		
 		if not path:
 			if not self.projectfile:
-				raise "No save path specified!"
+				raise Exception("No save path specified!")
 			path = self.projectfile
 		
 		if not self.audio_path:
@@ -663,13 +729,13 @@ class Project(gobject.GObject):
 			
 		if os.path.exists(self.audio_path):
 			if not os.path.isdir(self.audio_path):
-				raise "Audio save location is not a directory"
+				raise Exception("Audio save location is not a directory")
 		else:
 			os.mkdir(self.audio_path)
 		
 		if os.path.exists(self.levels_path):
 			if not os.path.isdir(self.levels_path):
-				raise "Levels save location is not a directory"
+				raise Exception("Levels save location is not a directory")
 		else:
 			os.mkdir(self.levels_path)
 		
@@ -680,13 +746,22 @@ class Project(gobject.GObject):
 		self.transportMode = self.transport.mode
 		
 		if not backup:
-			self.unsavedChanges = False
+			self.__unsavedChanges = False
 			#purge main undo stack so that it will not prompt to save on exit
 			self.__savedUndoStack.extend(self.__undoStack)
 			self.__undoStack = []
 			#purge savedRedoStack so that it will not prompt to save on exit
 			self.__redoStack.extend(self.__savedRedoStack)
 			self.__savedRedoStack = []
+			
+			# delete the incremental file since its all safe on disk now
+			basepath, ext = os.path.splitext(self.projectfile)
+			incr_path = basepath + self.INCREMENTAL_SAVE_EXT
+			try:
+				if os.path.exists(incr_path):
+					os.remove(incr_path)
+			except OSError:
+				Globals.debug("Removal of .incremental failed! Next load we will try to restore unrestorable state!")
 		
 		doc = xml.Document()
 		head = doc.createElement("JokosherProject")
@@ -701,6 +776,12 @@ class Project(gobject.GObject):
 		         "transportMode", "bpm", "meter_nom", "meter_denom", "projectfile"]
 		
 		Utils.StoreParametersToXML(self, doc, params, items)
+		
+		notesNode = doc.createElement("Notes")
+		head.appendChild(notesNode)
+		
+		# use repr() because XML will not preserve whitespace charaters such as \n and \t.
+		notesNode.setAttribute("text", repr(self.notes))
 			
 		undo = doc.createElement("Undo")
 		head.appendChild(undo)
@@ -736,11 +817,98 @@ class Project(gobject.GObject):
 		self.emit("undo")
 	
 	#_____________________________________________________________________
+	
+	def SaveIncrementalAction(self, action):
+		if self.isDoingIncrementalRestore:
+			return
+		
+		if self.__performingUndo or self.__performingRedo:
+			return
+		
+		path, ext = os.path.splitext(self.projectfile)
+		filename = path + self.INCREMENTAL_SAVE_EXT
+		
+		if self.hasDoneIncrementalSave:
+			incr_file = open(filename, "a")
+		else:
+			# if we haven't performed an incremental save yet,
+			# the existing .incremental file is old, so overwrite it.
+			incr_file = open(filename, "w")
+			self.hasDoneIncrementalSave = True
+		
+		incr_file.write(action.StoreToString())
+		incr_file.write(self.INCREMENTAL_SAVE_DELIMITER)
+		
+		incr_file.close()
+		
+		self.SetUnsavedChanges()
+		self.emit("incremental-save")
+	
+	#_____________________________________________________________________
+	
+	def CanDoIncrementalRestore(self):
+		path, ext = os.path.splitext(self.projectfile)
+		filename = path + self.INCREMENTAL_SAVE_EXT
+		return os.path.exists(filename)	
+	
+	#_____________________________________________________________________
+	
+	def DoIncrementalRestore(self):
+		"""
+		Loads all the actions from the .incremental file and executes them
+		to restore the project's state.
+		"""
+		
+		if self.hasDoneIncrementalSave:
+			Globals.debug("Cannot do incremental restore after incremental save.")
+			return False
+		
+		path, ext = os.path.splitext(self.projectfile)
+		filename = path + self.INCREMENTAL_SAVE_EXT
+		
+		save_action_list = []
+		
+		if os.path.isfile(filename):
+			incr_file = open(filename, "r")
+			filetext = incr_file.read()
+			incr_file.close()
+			for incr_xml in filetext.split(self.INCREMENTAL_SAVE_DELIMITER):
+				incr_xml = incr_xml.strip()
+				if incr_xml:
+					incr_action = IncrementalSave.LoadFromString(incr_xml)
+					save_action_list.append(incr_action)
+		
+		self.isDoingIncrementalRestore = True
+		try:
+			IncrementalSave.FilterAndExecuteAll(save_action_list, self)
+		except:
+			Globals.debug("Exception while restoring incremental save.",
+						"Project state is surely out of sync with .incremental file")
+			raise
+		
+		# set hasDoneIncrementSave to True because project is now in sync with .incremental file
+		# i.e. we don't have to destory the .incremental file because the states match up.
+		self.hasDoneIncrementalSave = True
+		self.isDoingIncrementalRestore = False
+		return True
+		
+	#_____________________________________________________________________
 
 	def CloseProject(self):
 		"""
 		Closes down this Project.
 		"""
+		
+		# when closing the file, the user chooses to either save, or discard
+		# in either case, we don't need the incremental save file anymore
+		path, ext = os.path.splitext(self.projectfile)
+		filename = path + self.INCREMENTAL_SAVE_EXT
+		try:
+			if os.path.exists(filename):
+				os.remove(filename)
+		except OSError:
+			Globals.debug("Removal of .incremental failed! Next load we will try to restore unrestorable state!")
+		
 		for file in self.deleteOnCloseAudioFiles:
 			if os.path.exists(file):
 				Globals.debug("Deleting copied audio file:", file)
@@ -769,6 +937,10 @@ class Project(gobject.GObject):
 			self.__savedUndo = False
 			
 		self.__performingUndo = False
+		
+		# __performingUndo must be False for project to log
+		inc = IncrementalSave.Undo()
+		self.SaveIncrementalAction(inc)
 	
 	#_____________________________________________________________________
 	
@@ -789,6 +961,10 @@ class Project(gobject.GObject):
 			self.ExecuteAction(cmd)
 			
 		self.__performingRedo = False
+		
+		# __performingRedo must be False for project to log
+		inc = IncrementalSave.Redo()
+		self.SaveIncrementalAction(inc)
 
 	#_____________________________________________________________________
 	
@@ -816,7 +992,7 @@ class Project(gobject.GObject):
 				self.__savedRedoStack = []
 				#since there is no other record that something has 
 				#changed after savedRedoStack is purged
-				self.unsavedChanges = True
+				self.__unsavedChanges = True
 		self.emit("undo")
 	
 	#_____________________________________________________________________
@@ -837,19 +1013,25 @@ class Project(gobject.GObject):
 	
 	def CheckUnsavedChanges(self):
 		"""
-		Uses boolean self.unsavedChanges and Undo/Redo to 
+		Uses boolean self.__unsavedChanges and Undo/Redo to 
 		determine if the program needs to save anything on exit.
 		
 		Return:
 			True -- there's unsaved changes, undoes or redoes
 			False -- the Project can be safely closed.
 		"""
-		return self.unsavedChanges or \
+		return self.__unsavedChanges or \
 			len(self.__undoStack) > 0 or \
 			len(self.__savedRedoStack) > 0
 	
 	#_____________________________________________________________________
 	
+	def SetUnsavedChanges(self):
+		self.__unsavedChanges = True
+		self.emit("undo") 
+		
+	#_____________________________________________________________________
+	
 	def CanPerformUndo(self):
 		"""
 		Whether it's possible to perform an undo operation.
@@ -885,28 +1067,39 @@ class Project(gobject.GObject):
 		newUndoAction = self.NewAtomicUndoAction()
 		for cmdList in reversed(undoAction.GetUndoCommands()):
 			obj = cmdList[0]
-			target_object = None
-			if obj[0] == "P":		# Check if the object is a Project
-				target_object = self
-			elif obj[0] == "I":		# Check if the object is an Instrument
-				id = int(obj[1:])
-				target_object = [x for x in self.instruments if x.id==id][0]
-			elif obj[0] == "E":		# Check if the object is an Event
-				id = int(obj[1:])
-				for instr in self.instruments:
-					# First of all see if it's alive on an instrument
-					n = [x for x in instr.events if x.id==id]
-					if not n:
-						# If not, check the graveyard on each instrument
-						n = [x for x in instr.graveyard if x.id==id]
-					if n:
-						target_object = n[0]
-						break
+			target_object = self.JokosherObjectFromString(obj)
 			
 			getattr(target_object, cmdList[1])(_undoAction_=newUndoAction, *cmdList[2])
 
 	#_____________________________________________________________________
 	
+	def JokosherObjectFromString(self, string):
+		"""
+		Converts a string used to serialize references to Project, Instrument
+		and Event instances into a reference to the actual object.
+		
+		Parameters:
+			string -- The string to convert such as "P" for project or "I2" for instrument with ID equal to 2.
+		"""
+		if string[0] == "P":		# Check if the object is a Project
+			return self
+		elif string[0] == "I":		# Check if the object is an Instrument
+			id = int(string[1:])
+			for instr in self.instruments:
+				if instr.id == id:
+					return instr
+		elif string[0] == "E":		# Check if the object is an Event
+			id = int(string[1:])
+			for instr in self.instruments:
+				for event in instr.events:
+					if event.id == id:
+						return event
+				for event in instr.graveyard:
+					if event.id == id:
+						return event
+				
+	#_____________________________________________________________________
+	
 	@UndoSystem.UndoCommand("SetBPM", "temp")
 	def SetBPM(self, bpm):
 		"""
@@ -956,16 +1149,19 @@ class Project(gobject.GObject):
 		they are all appended to the undo stack as a single atomic action.
 		
 		Parameters:
-			instrTuples -- a list of tuples containing name, type and pixbuf
+			instrTuples -- a list of tuples containing name and type
 					that will be passed to AddInstrument().
 			
 		Returns:
-			A list of IDs of the added Instruments.
+			A list of the added Instruments.
 		"""
 		
 		undoAction = self.NewAtomicUndoAction()
-		for name, type, pixbuf in instrTuples:
-			self.AddInstrument(name, type, pixbuf, _undoAction_=undoAction)
+		instrList = []
+		for name, type in instrTuples:
+			instr = self.AddInstrument(name, type, _undoAction_=undoAction)
+			instrList.append(instr)
+		return instrList
 	
 	#_____________________________________________________________________
 	
@@ -986,7 +1182,7 @@ class Project(gobject.GObject):
 	#_____________________________________________________________________
 	
 	@UndoSystem.UndoCommand("DeleteInstrument", "temp")
-	def AddInstrument(self, name, type, pixbuf):
+	def AddInstrument(self, name, type):
 		"""
 		Adds a new instrument to the Project and returns the ID for that instrument.
 		
@@ -997,12 +1193,11 @@ class Project(gobject.GObject):
 		Parameters:
 			name -- name of the instrument.
 			type -- type of the instrument.
-			pixbuf -- image object corresponding to the instrument.
 			
 		Returns:
-			ID of the added Instrument.
+			The created Instrument object.
 		"""
-			
+		pixbuf = Globals.getCachedInstrumentPixbuf(type)
 		instr = Instrument.Instrument(self, name, type, pixbuf)
 		if len(self.instruments) == 0:
 			#If this is the first instrument, arm it by default
@@ -1113,7 +1308,7 @@ class Project(gobject.GObject):
 			undoAction = self.NewAtomicUndoAction()
 		
 		name, type, pixbuf, path = [x for x in Globals.getCachedInstruments() if x[1] == "audiofile"][0]
-		instr = self.AddInstrument(name, type, pixbuf, _undoAction_=undoAction)
+		instr = self.AddInstrument(name, type, _undoAction_=undoAction)
 		instr.AddEventsFromList(0, fileList, copyFile, undoAction)
 	
 	#_____________________________________________________________________
@@ -1209,13 +1404,15 @@ class Project(gobject.GObject):
 			if id in self.___id_list:
 				Globals.debug("Error: id", id, "already taken")
 			else:
-				self.___id_list.append(id)
+				if reserve:
+					self.___id_list.append(id)
 				return id
 				
 		counter = 0
 		while True:
 			if not counter in self.___id_list:
-				self.___id_list.append(counter)
+				if reserve:
+					self.___id_list.append(counter)
 				return counter
 			counter += 1
 	
@@ -1287,17 +1484,19 @@ class Project(gobject.GObject):
 
 	#_____________________________________________________________________
 
-	def SetClickEnabled(self, active):
+	def SetClickTrackVolume(self, value):
 		"""
 		Unmutes and enables the click track.
 		
 		Parameters:
-			active -- If True, the click track will be activated, otherwise it will be disabled.
+			value -- The volume of the click track between 0.0 and 1.0
 		"""
-		if self.clickEnabled != active:
-			self.clickTrackVolume.set_property("mute", not active)
-			self.clickEnabled = active
-			self.emit("click-track")
+		if self.clickVolumeValue != value:
+			self.clickTrackVolume.set_property("mute", (value < 0.01))
+			# convert the 0.0 to 1.0 range to 0.0 to 2.0 range (to let the user make it twice as loud)
+			self.clickTrackVolume.set_property("volume", value * 2)
+			self.clickVolumeValue = value
+			self.emit("click-track", value)
 
 	#_____________________________________________________________________
 
@@ -1347,38 +1546,33 @@ class Project(gobject.GObject):
 			return self.masterSink
 		
 		self.currentSinkString = sinkString
-		sinkElement = None
+		sinkBin = None
 		
-		if sinkString == "alsasink":
-			sinkElement = gst.element_factory_make("alsasink")
-			#Set the alsa device for audio output
-			outdevice = Globals.settings.playback["devicecardnum"]
-			if outdevice == "default":
-				try:
-					# Select first output device as default to avoid a GStreamer bug which causes
-					# large amounts of latency with the ALSA 'default' device.
-					outdevice = AlsaDevices.GetAlsaList("playback").keys()[1]
-				except:
-					pass
-			Globals.debug("Output device: %s" % outdevice)
-			sinkElement.set_property("device", outdevice)
-			Globals.debug("Using alsasink for audio output")
-		
-		elif sinkString != "autoaudiosink":
-			try:
-				sinkElement = gst.parse_bin_from_description(sinkString, True)
-			except gobject.GError:
-				Globals.debug("Parsing failed: %s" % sinkString)
-			else:
-				Globals.debug("Using custom pipeline for audio sink: %s" % sinkString)
-		
-		if not sinkElement:
-			# if a sink element has not yet been created, autoaudiosink is our last resort
-			sinkElement = gst.element_factory_make("autoaudiosink")
+		try:
+			sinkBin = gst.parse_bin_from_description(sinkString, True)
+		except gobject.GError:
+			Globals.debug("Parsing failed: %s" % sinkString)
+			# autoaudiosink is our last resort
+			sinkBin = gst.element_factory_make("autoaudiosink")
 			Globals.debug("Using autoaudiosink for audio output")
+		else:
+			Globals.debug("Using custom pipeline for audio sink: %s" % sinkString)
 			
-		return sinkElement
+			sinkElement = sinkBin.sinks().next()
+			if hasattr(sinkElement.props, "device"):
+				outdevice = Globals.settings.playback["device"]
+				Globals.debug("Output device: %s" % outdevice)
+				sinkElement.set_property("device", outdevice)
 		
+		return sinkBin
+		
+	#____________________________________________________________________	
+	
+	def OnCaptureBackendChange(self):
+		for instr in self.instruments:
+			instr.input = None
+			instr.inTrack = -1
+	
 	#____________________________________________________________________	
 	
 	def GetInputFilenames(self):
@@ -1397,5 +1591,27 @@ class Project(gobject.GObject):
 		
 	#____________________________________________________________________	
 
+	def SetName(self, name):
+		if self.name != name:
+			self.name = name
+			inc = IncrementalSave.SetName(name)
+			self.SaveIncrementalAction(inc)
+				
+	#____________________________________________________________________	
 	
+	def SetAuthor(self, author):
+		if self.author != author:
+			self.author = author
+			inc = IncrementalSave.SetAuthor(author)
+			self.SaveIncrementalAction(inc)
+	
+	#____________________________________________________________________	
+	
+	def SetNotes(self, notes):
+		if self.notes != notes:
+			self.notes = notes
+			inc = IncrementalSave.SetNotes(notes)
+			self.SaveIncrementalAction(inc)
+	
+	#____________________________________________________________________		
 #=========================================================================
