@@ -13,6 +13,7 @@ import gtk
 import InstrumentViewer
 import TimeLineBar
 import Globals
+import ui.MessageArea as MessageArea
 from gettext import gettext as _
 import urllib
 
@@ -27,8 +28,6 @@ class RecordingView(gtk.Frame):
 	""" GTK widget name """
 	__gtype_name__ = 'RecordingView'
 	
-	""" Width, in pixel, for the instrument headers """
-	INSTRUMENT_HEADER_WIDTH = 150
 	""" How far in you are allowed to zoom. """
 	ZOOM_MAX_SCALE = 100.0
 	""" How for out you are allowed to zoom. 
@@ -45,6 +44,9 @@ class RecordingView(gtk.Frame):
 						URI_DRAG_TYPE ),		# Use the custom number
 						("text/plain", 0, URI_DRAG_TYPE) # so drags from Firefox work
 						]
+	
+	"""The number of seconds shown after the end of the last event"""
+	EXTRA_SCROLL_TIME = 25
 
 	#_____________________________________________________________________
 
@@ -62,7 +64,13 @@ class RecordingView(gtk.Frame):
 		self.project = project
 		self.mainview = mainview
 		self.small = small
-		self.timelinebar = TimeLineBar.TimeLineBar(self.project, self, mainview)
+		self.timelinebar = TimeLineBar.TimeLineBar(self.project, mainview)
+		
+		self.gstreamerErrorMessages = []
+		self.errorMessageArea = None
+		self.moreErrorsMessageArea = None
+		
+		self.restoreMessageArea = None
 		
 		## create darker workspace box
 		self.eventBox = gtk.EventBox()
@@ -82,15 +90,25 @@ class RecordingView(gtk.Frame):
 		self.instrumentWindow.child.set_shadow_type(gtk.SHADOW_NONE)
 		self.views = []	
 		
+		self.header_size_group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+		self.header_size_group.add_widget(self.timelinebar.GetHeaderWidget())
+		
 		self.hb = gtk.HBox()
 		self.hb.set_spacing(6)
+		self.hb.set_border_width(6)
 		self.vbox.pack_end(self.hb, False, False)
-		self.al = gtk.Alignment(0, 0, 1, 1)
+		self.vbox.pack_end(gtk.HSeparator(), False, False)
+		
+		self.zoom_hb = gtk.HBox()
+		self.zoom_hb.set_spacing(6)
+		self.zoom_hb.set_border_width(0)
+		self.header_size_group.add_widget(self.zoom_hb)
+		
 		self.scrollRange = gtk.Adjustment()
-		sb = gtk.HScrollbar(self.scrollRange)
-		self.al.add(sb)
-		self.al.set_padding(0, 0, 0, 0)
-		self.hb.pack_start(self.al)
+		self.scrollBar = gtk.HScrollbar(self.scrollRange)
+		
+		self.hb.pack_start(self.zoom_hb, False, False)
+		self.hb.pack_start(self.scrollBar, True, True)
 		
 		self.lastzoom = 0
 		
@@ -121,27 +139,28 @@ class RecordingView(gtk.Frame):
 		self.zoomtip.set_tip(self.outbutton, _("Zoom out timeline"), None)
 		self.outbutton.connect("clicked", self.OnZoomOut)
 
-		self.hb.pack_start( self.outbutton, False, False)
-		self.hb.pack_start( self.zoomSlider, False, False)
-		self.hb.pack_start( self.inbutton, False, False)
+		self.zoom_hb.pack_start( self.outbutton, False, False)
+		self.zoom_hb.pack_start( self.zoomSlider, False, False)
+		self.zoom_hb.pack_start( self.inbutton, False, False)
 		
-		self.extraScrollTime = 25
 		self.centreViewOnPosition = False
 		self.scrollRange.lower = 0
 		self.scrollRange.upper = 100
 		self.scrollRange.value = 0
 		self.scrollRange.step_increment = 1
 		
-		sb.connect("value-changed", self.OnScroll)
+		self.scrollBar.connect("value-changed", self.OnScroll)
 		self.connect("expose-event", self.OnExpose)
 		self.connect("button_release_event", self.OnExpose)
 		self.connect("button_press_event", self.OnMouseDown)
-		self.connect("size-allocate", self.OnAllocate)
 		
 		#connect to the project signals
+		self.project.connect("gst-bus-error", self.OnProjectGstError)
+		self.project.connect("incremental-save", self.OnProjectIncSave)
 		self.project.connect("instrument::added", self.OnInstrumentAdded)
 		self.project.connect("instrument::reordered", self.OnInstrumentReordered)
 		self.project.connect("instrument::removed", self.OnInstrumentRemoved)
+		self.project.connect("view-start", self.OnViewStartChanged)
 		
 		self.vbox.drag_dest_set(	gtk.DEST_DEFAULT_DROP,
 									self.DRAG_TARGETS, 
@@ -160,6 +179,22 @@ class RecordingView(gtk.Frame):
 			self.outbutton.hide()
 			self.zoomSlider.hide()
 		
+		if self.project.CanDoIncrementalRestore():
+			message = _("Would you like to restore the current project?")
+			details = _("A crash was detected and changes to your project were not saved.\nIf you would like, you can attempt to recover these lost changes.")
+			
+			msg_area = MessageArea.MessageArea()
+			msg_area.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
+			msg_area.add_stock_button_with_text(_("_Restore Project"), gtk.STOCK_APPLY, gtk.RESPONSE_OK)
+			msg_area.set_text_and_icon(gtk.STOCK_DIALOG_QUESTION, message, details)
+			
+			msg_area.connect("response", self.OnRestoreMessageAreaResponse, msg_area)
+			msg_area.connect("close", self.OnRestoreMessageAreaClose, msg_area)
+			
+			self.vbox.pack_end(msg_area, False, False)
+			msg_area.show()
+			self.restoreMessageArea = msg_area
+		
 	#_____________________________________________________________________
 
 	def OnExpose(self, widget, event):
@@ -188,45 +223,34 @@ class RecordingView(gtk.Frame):
 		## use the new colormap
 		self.eventBox.set_style(stcp)
 		
-		# calculate scrollable width - allow 4 pixels for borders
-		self.scrollRange.page_size = (self.allocation.width - Globals.INSTRUMENT_HEADER_WIDTH - 4) / self.project.viewScale
+		# calculate scrollable width (scroll bar should always be same width as viewable area)
+		self.scrollRange.page_size = (self.scrollBar.allocation.width) / self.project.viewScale
 		self.scrollRange.page_increment = self.scrollRange.page_size
 		# add EXTRA_SCROLL_TIME extra seconds
-		length = self.project.GetProjectLength() + self.extraScrollTime
+		length = self.project.GetProjectLength() + self.EXTRA_SCROLL_TIME
 		self.scrollRange.upper = length
 		
 		if self.centreViewOnPosition:  
 			self.centreViewOnPosition = False  
 			#set the view to be centred over the playhead  
 			start = self.project.transport.GetPosition() - (self.scrollRange.page_size / 2)
-			self.SetViewPosition(start)
+			self.project.SetViewStart(start)
 		# Need to adjust project view start if we are zooming out
 		# and the end of the project is now before the end of the page.
 		# Project end will be at right edge unless the start is also on 
 		# screen, in which case the start will be at the left.
 		elif self.project.viewStart + self.scrollRange.page_size > length:
-			self.SetViewPosition(length - self.scrollRange.page_size)
+			self.project.SetViewStart(length - self.scrollRange.page_size)
 		
 		#check the min zoom value (based on project length)
-		pixelSize = self.allocation.width - Globals.INSTRUMENT_HEADER_WIDTH - 4	# four pixels to account for borders
+		# (scroll bar should always be same width as viewable area)
+		pixelSize = self.scrollBar.allocation.width 
 		minScale = pixelSize / length
 		self.zoomSlider.set_range(minScale, self.ZOOM_MAX_SCALE)
 		if self.zoomSlider.get_value() < minScale:
 			self.zoomSlider.set_value(minScale)
 		
 	#_____________________________________________________________________
-
-	def OnAllocate(self, widget, allocation):
-		"""
-		Callback for "size-allocate" signal.
-		
-		Parameters:
-			widget -- reserved for GTK callbacks, don't use it explicitly.
-			allocation -- new allocation value to be set.
-		"""
-		self.allocation = allocation
-		
-	#_____________________________________________________________________
 	
 	def OnInstrumentAdded(self, project, instrument):
 		"""
@@ -240,12 +264,10 @@ class RecordingView(gtk.Frame):
 		
 		#Add it to the views
 		self.views.append((instrument.id, instrViewer))
-		instrViewer.headerAlign.connect("size-allocate", self.UpdateSize)
+		self.header_size_group.add_widget(instrViewer.GetHeaderWidget())
 		
 		self.instrumentBox.pack_start(instrViewer, False, False)
-		instrViewer.show_all()
-		
-		self.ForceUpdateSize()
+		instrViewer.show()
 	
 	#_____________________________________________________________________
 	
@@ -264,8 +286,6 @@ class RecordingView(gtk.Frame):
 				instrViewer.Destroy()
 				self.views.remove((ID, instrViewer))
 				break
-				
-		self.ForceUpdateSize()
 	
 	#_____________________________________________________________________
 	
@@ -288,35 +308,126 @@ class RecordingView(gtk.Frame):
 	
 	#_____________________________________________________________________
 	
-	def ForceUpdateSize(self):
+	def OnProjectGstError(self, project, error, details):
 		"""
-		Force and update of a header size to match the size of the first instrument viewer.
+		Callback for when the project sends a gstreamer error message
+		from the pipeline.
+		
+		Parameters:
+			project -- The project instance that send the signal.
+			error -- The type of error that occurred as a string.
+			details -- A string with more information about the error.
 		"""
-		if self.views:
-			self.UpdateSize(None, self.views[0][1].headerAlign.get_allocation())
-		else:
-			self.UpdateSize(None, None)
-	
+		if not error:
+			error = _("A Gstreamer error has occurred")
+		#outrostring = _("Please report this error to the Jokosher developers at:\nhttps://bugs.launchpad.net/jokosher\nYou can also get help from:\nhttp://www.jokosher.org/forums/")
+		
+		self.gstreamerErrorMessages.append((error, details))
+		# invalidate the second error pane since the number it displays has changed
+		if self.moreErrorsMessageArea:
+			self.vbox.remove(self.moreErrorsMessageArea)
+			self.moreErrorsMessageArea = None
+		self.ReloadErrorMessages()
+		
 	#_____________________________________________________________________
 	
-	def UpdateSize(self, widget=None, size=None):
-		"""
-		This method will re-align the timeline and scrollbars
-		with the start of the event lane since the instrument 
-		width may have been altered.
-		"""
-		#find the width of the instrument headers (they should all be the same size)
-		if size:
-			tempWidth = size.width
-		else:
-			tempWidth = self.INSTRUMENT_HEADER_WIDTH
+	def CreateDefaultErrorPane(self, error, details):
+		message = _("A GStreamer error has occurred.")
 		
-		#set it to the globals class
-		Globals.INSTRUMENT_HEADER_WIDTH = tempWidth
+		details = "\n".join((error, details))
 		
-		#align timeline and scrollbar
-		self.timelinebar.UpdateSize()
-		self.al.set_padding(0, 0, tempWidth, 0)
+		msg_area = MessageArea.MessageArea()
+		msg_area.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
+		msg_area.set_text_and_icon(gtk.STOCK_DIALOG_ERROR, message, details)
+		
+		msg_area.connect("response", self.OnMessageAreaReponse, msg_area)
+		msg_area.connect("close", self.OnMessageAreaClose, msg_area)
+		
+		return msg_area
+	
+	#_____________________________________________________________________
+		
+	def ReloadErrorMessages(self):
+		num_errors = len(self.gstreamerErrorMessages)
+		
+		if self.errorMessageArea:
+			self.vbox.remove(self.errorMessageArea)
+		if self.moreErrorsMessageArea:
+			self.vbox.remove(self.moreErrorsMessageArea)
+		
+		if num_errors == 0:
+			return
+		
+		
+		if not self.errorMessageArea:
+			error, details = self.gstreamerErrorMessages[0]
+			self.errorMessageArea = self.CreateDefaultErrorPane(error, details)
+		
+		
+		if num_errors == 2 and not self.moreErrorsMessageArea:
+			error, details = self.gstreamerErrorMessages[1]
+			self.moreErrorsMessageArea = self.CreateDefaultErrorPane(error, details)
+			
+		elif num_errors > 2 and not self.moreErrorsMessageArea:
+			multi_error = _("There are %(number)d more errors not shown.")
+			multi_error %= {"number" : num_errors - 1 }
+			
+			info = _("If this problem persists consider reporting a bug using the link in the help menu.")
+			
+			msg_area = MessageArea.MessageArea()
+			msg_area.add_stock_button_with_text(_("_Close All"), gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
+			msg_area.add_button(_("_Show All"), gtk.RESPONSE_YES)
+			msg_area.set_text_and_icon(gtk.STOCK_DIALOG_ERROR, multi_error, info)
+			
+			msg_area.connect("response", self.OnMessageAreaReponse, msg_area)
+			msg_area.connect("close", self.OnMessageAreaClose, msg_area)
+			
+			self.moreErrorsMessageArea = msg_area
+		
+		# now repack them in the correct order
+		if self.moreErrorsMessageArea:
+			self.vbox.pack_end(self.moreErrorsMessageArea, False, False)
+			self.moreErrorsMessageArea.show()
+			
+		self.vbox.pack_end(self.errorMessageArea, False, False)
+		self.errorMessageArea.show()
+		
+	#_____________________________________________________________________
+	
+	def OnMessageAreaClose(self, widget, message_area):
+		if message_area is self.errorMessageArea:
+			self.gstreamerErrorMessages.pop(0)
+		elif message_area is self.moreErrorsMessageArea:
+			self.gstreamerErrorMessages = []
+			
+		if self.errorMessageArea:
+			self.vbox.remove(self.errorMessageArea)
+			self.errorMessageArea = None
+		if self.moreErrorsMessageArea:
+			self.vbox.remove(self.moreErrorsMessageArea)
+			self.moreErrorsMessageArea = None
+			
+		self.ReloadErrorMessages()
+				
+	#_____________________________________________________________________
+	
+	def OnMessageAreaReponse(self, widget, response_id, message_area):
+		if response_id == gtk.RESPONSE_CLOSE:
+			self.OnMessageAreaClose(widget, message_area)
+		if response_id == gtk.RESPONSE_YES:
+			# this is the response for Show All button
+			self.res = gtk.glade.XML(Globals.GLADE_PATH, "ErrorPaneDialog")
+			dialog = self.res.get_widget("ErrorPaneDialog")
+			main_vbox = self.res.get_widget("MainVBox")
+			for error, details in self.gstreamerErrorMessages:
+				msg_area = MessageArea.MessageArea()
+				#msg_area.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
+				msg_area.set_text_and_icon(gtk.STOCK_DIALOG_ERROR, error, details)
+				main_vbox.pack_start(msg_area, False, False)
+				
+			dialog.connect("response", lambda dlg, resp: dlg.destroy())
+			dialog.show_all()
+			
 	
 	#_____________________________________________________________________
 	
@@ -407,22 +518,16 @@ class RecordingView(gtk.Frame):
 			return True
 		
 	#_____________________________________________________________________
-
-	def SetViewPosition(self, position):
+	
+	def OnViewStartChanged(self, project):
 		"""
-		Moves the view so that the given position is the leftmost side
-		of the viewable area for scrolling, etc.
+		Callback for when the project notifies that the
+		viewable start position has changed.
 		
 		Parameters:
-			position -- the new position to set.
+			project -- The project instance that send the signal.
 		"""
-		length = self.project.GetProjectLength() + self.extraScrollTime 
-		#check if its over the project length
-		start = min(length - self.scrollRange.page_size, position)
-		#check if its under zero (do this after checking the project length, because if the project length is 0 it will go under)
-		start = max(0, start)
-		self.scrollRange.value = start
-		self.project.SetViewStart(start)
+		self.scrollRange.value = project.viewStart
 	
 	#_____________________________________________________________________
 	
@@ -530,5 +635,24 @@ class RecordingView(gtk.Frame):
 			self.zoomSlider.show()
 			
 	#____________________________________________________________________	
-
+	
+	def OnRestoreMessageAreaClose(self, widget=None, msg_area=None):
+		if self.restoreMessageArea:
+			self.vbox.remove(self.restoreMessageArea)
+			self.restoreMessageArea = None
+	
+	#____________________________________________________________________
+	
+	def OnRestoreMessageAreaResponse(self, widget, response_id, msg_area):
+		if response_id == gtk.RESPONSE_OK:
+			self.project.DoIncrementalRestore()
+		
+		self.OnRestoreMessageAreaClose()
+	
+	#____________________________________________________________________
+	
+	def OnProjectIncSave(self, project):
+		self.OnRestoreMessageAreaClose()
+	
+	#____________________________________________________________________
 #=========================================================================
