@@ -10,65 +10,68 @@
 #-------------------------------------------------------------------------------
 
 import gtk
-import TimeLine
+import gobject
 import gettext
 import os
+import TimeLine
 import Globals
-import gobject
+import ui.EventLaneHSeparator as EventLaneHSeparator
 
 # for the time signature combo
 from math import log
 
 _=gettext.gettext
 
-class TimeLineBar(gtk.Frame):
+class TimeLineBar(gtk.HBox):
 	"""
 	This class contains the TimeLine widget as well as the click track button and the bpm label in a gtk.Frame widget.
 	"""
 	#_____________________________________________________________________
 	
-	def __init__(self, project, projectview, mainview):
+	def __init__(self, project, mainview):
 		"""
 		Creates a new instance of TimeLineBar
 		
 		Parameters:
 			project -- reference to Project (Project.py).
-			projectview -- reference to RecordingView (RecordingView.py).
 			mainview -- reference to MainApp (JokosherApp.py).
 		"""
-		gtk.Frame.__init__(self)
+		gtk.HBox.__init__(self)
 		
 		self.project = project
-		self.projectview = projectview
 		self.mainview = mainview
-		self.timeline = TimeLine.TimeLine(self.project, self, mainview)
+		self.timeline = TimeLine.TimeLine(self.project, mainview)
 		self.Updating = False
 		
-		self.bgColor = "#73d216"
-		self.bgActiveColor = "#81eb17"
-		self.fontColor = "#0b410b"
-		
 		# add click / bpm / signature box
-		self.clickbutton = gtk.ToggleButton()
+		self.clickbutton = gtk.VolumeButton()
+		self.clickbutton.set_value(0)
+		self.clickbutton.set_relief(gtk.RELIEF_NORMAL)
+		self.clickbutton.set_property("size", gtk.ICON_SIZE_BUTTON)
+		
+		self.clickbutton_metronome_image = gtk.Image()
+		self.clickbutton_metronome_image.set_from_file(os.path.join(Globals.IMAGE_PATH, "icon_click.png"))
+		# use get_child() not get_image here because GtkScaleButton
+		# uses gtk_container_add() in its internal implementation
+		self.clickbutton_volume_image = self.clickbutton.get_child()
+		self.clickbutton.set_image(self.clickbutton_metronome_image)
+		
+		image_size_group = gtk.SizeGroup(gtk.SIZE_GROUP_BOTH)
+		image_size_group.add_widget(self.clickbutton_metronome_image)
+		image_size_group.add_widget(self.clickbutton_volume_image)
+		
 		self.clicktip = gtk.Tooltips()
-		clickimg = gtk.Image()
-		clickimg.set_from_file(os.path.join(Globals.IMAGE_PATH, "icon_click.png"))
-		self.clickbutton.set_image(clickimg)
-
-		self.clicktip.set_tip(self.clickbutton, _("Turn click track on"), None)
-		self.clickbutton.connect("toggled", self.OnClick)
+		self.clicktip.set_tip(self.clickbutton, _("Adjust volume of click track"), None)
 					
 		self.bpmeventbox = gtk.EventBox()
-		self.bpmeventbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.bgColor))
 		self.bpmframe = gtk.Frame()
 		self.bpmeventtip = gtk.Tooltips()
 		self.bpmeventtip.set_tip(self.bpmeventbox, _("Beats per minute"), None)
 		self.bpmframe.set_shadow_type(gtk.SHADOW_ETCHED_OUT)
-		self.bpmframe.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.bgColor))
 		
 		self.bpmlabel = gtk.Label()
 		self.bpmlabel.set_use_markup(True)
-		self.bpmlabel.set_markup("<span foreground='%s'><b>%s</b></span>" % (self.fontColor, self.project.bpm))
+		self.bpmlabel.set_markup("<b>%s</b>" % (self.project.bpm))
 		self.bpmlabel.set_padding(5, 5)
 		self.bpmeventbox.add(self.bpmlabel)
 		self.bpmframe.add(self.bpmeventbox)
@@ -78,26 +81,30 @@ class TimeLineBar(gtk.Frame):
 		self.bpmedit.set_increments(1, 5)
 		self.bpmedit.set_value(self.project.bpm)
 		self.bpmedit.connect("activate", self.OnAcceptEditBPM)
+		self.bpmedit.connect("key_press_event", self.OnEditBPMKey)
+		self.bpmedit.connect_after("button-release-event", self.OnEditBPMClick)
 
 		self.sigeventbox = gtk.EventBox()
-		self.sigeventbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.bgColor))
 		self.sigeventtip = gtk.Tooltips()
 		self.sigeventtip.set_tip(self.sigeventbox, _("Time signature"), None)
 		self.sigframe = gtk.Frame()
 		self.sigframe.set_shadow_type(gtk.SHADOW_ETCHED_OUT)
-		self.sigframe.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.bgColor))
-
+		
 		self.siglabel = gtk.Label()
 		self.siglabel.set_use_markup(True)
-		self.siglabel.set_markup("<span foreground='%s'><b>%d/%d</b></span>" % (self.fontColor, self.project.meter_nom, self.project.meter_denom))
+		self.siglabel.set_markup("<b>%d/%d</b>" % (self.project.meter_nom, self.project.meter_denom))
 		self.siglabel.set_padding(5, 5)
 		self.sigeventbox.add(self.siglabel)
 		self.sigframe.add(self.sigeventbox)
 		self.sigeditPacked = False
 
 		# set events
+		self.clickbutton.connect("value-changed", self.OnClickTrackVolume)
+		self.clickbutton.connect("enter_notify_event", self.OnClickButtonEnter)
+		self.clickbutton.connect("leave_notify_event", self.OnClickButtonEnter)
+		
 		self.bpmeventbox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
-		self.bpmeventbox.connect("button_press_event", self.OnEditBPM)
+		self.bpmeventbox.connect("button_release_event", self.OnEditBPM)
 		self.bpmeventbox.connect("enter_notify_event", self.OnMouseMoveBPM)
 		self.bpmeventbox.connect("leave_notify_event", self.OnMouseMoveBPM)
 		
@@ -121,60 +128,26 @@ class TimeLineBar(gtk.Frame):
 		self.headerhbox.pack_start(self.bpmframe, True, True)
 		self.headerhbox.pack_start(self.sigframe, True, True)
 		
-		self.hbox = gtk.HBox()
-		self.alignment = gtk.Alignment(0, 0, 1.0, 1.0)
-		self.alignment.add(self.headerhbox)
-		self.hbox.pack_start(self.alignment, False, False)
-		self.add(self.hbox)
-		self.headerhbox.connect("check-resize", self.projectview.ForceUpdateSize)
-		self.connect("size-allocate", self.OnAllocate)
-		self.hbox.pack_start(self.timeline)	
-
-	#_____________________________________________________________________
-
-	def OnAllocate(self, widget, allocation):
-		"""
-		From:
-		http://www.moeraki.com/pygtkreference/pygtk2reference/class-gtkwidget.html#signal-gtkwidget--size-allocate
-		The "size-allocate" signal is emitted when widget is given a new space allocation.
+		self.headerVBox = gtk.VBox()
+		self.headerVBox.pack_start(self.headerhbox, True, True)
+		self.headerVBox.pack_start(gtk.HSeparator(), False, False)
 		
-		Parameters:
-			widget -- reserved for GTK callbacks, don't use it explicitly. 
-			allocation -- the position and size to be allocated to the widget.
-		"""
-		self.allocation = allocation
+		self.timelineVBox = gtk.VBox()
+		self.timelineVBox.pack_start(self.timeline, True, True)
+		separator = EventLaneHSeparator.EventLaneHSeparator(self.project, self.project.transport)
+		self.timelineVBox.pack_start(separator, False, False)
+		
+		self.pack_start(self.headerVBox, False, False)
+		self.pack_start(self.timelineVBox)	
 
 	#_____________________________________________________________________
 	
-	def UpdateSize(self):
-		""" 
-		Updates the size of the header box contents TimeLineBar, updating the values in the beats per
-		minute box and time signature box, as well as updating the click button
-		sensitivity and instrument header width.
+	def GetHeaderWidget(self):
 		"""
-		if self.Updating or not self.mainview.workspace:
-			return
-		
-		instrumentViews = self.mainview.workspace.recordingView.views
-
-		self.Updating = True
-		maxwidth = self.headerhbox.size_request()[0]
-
-		for ident, iv in instrumentViews:  #self.mainview.recording.views:
-			if iv.instrument in iv.mainview.project.instruments:
-				if iv.headerBox.size_request()[0] > maxwidth:
-					maxwidth = iv.headerBox.size_request()[0]
-
-		for ident, iv in instrumentViews:  #self.mainview.recording.views:
-			if iv.headerAlign.size_request()[0] != (maxwidth+2):
-				iv.ResizeHeader(maxwidth+2)
-		
-		Globals.INSTRUMENT_HEADER_WIDTH = maxwidth + 2
-		
-		self.alignment.set_padding(0, 0, 0, maxwidth - self.headerhbox.size_request()[0])
-		
-		self.Updating = False
-		
+			Returns the widget which is required to be aligned with the instrument headers.
+		"""
+		return self.headerVBox
+	
 	#_____________________________________________________________________
 	
 	def OnProjectBPMChange(self, project):
@@ -187,10 +160,7 @@ class TimeLineBar(gtk.Frame):
 		"""
 		#Do this outside the if statement so that it gets updated if someone else changes the bpm
 		self.bpmlabel.set_use_markup(True)
-		self.bpmlabel.set_markup("<span foreground='%s'><b>%d</b></span>" % (self.fontColor, self.project.bpm))
-		self.bpmeventbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.bgColor))
-		
-		self.projectview.UpdateSize()
+		self.bpmlabel.set_markup("<b>%d</b>" % (self.project.bpm))
 		
 	#_____________________________________________________________________
 	
@@ -203,30 +173,23 @@ class TimeLineBar(gtk.Frame):
 			project -- The project that send the signal.
 		"""
 		self.siglabel.set_use_markup(True)
-		self.siglabel.set_markup("<span foreground='%s'><b>%d/%d</b></span>" % (self.fontColor, self.project.meter_nom, self.project.meter_denom))
-		self.projectview.UpdateSize()
+		self.siglabel.set_markup("<b>%d/%d</b>" % (self.project.meter_nom, self.project.meter_denom))
 		
 	#_____________________________________________________________________
 	
-	def OnProjectClickTrackChange(self, project):
+	def OnProjectClickTrackChange(self, project, value):
 		"""
 		Callback for when the click track of the project it turned on
 		or shut off. This method will update the button and tooltip state.
 		
 		Parameters:
 			project -- The project that send the signal.
+			value -- The new value of the click track volume.
 		"""
 		self.Updating = True
-		self.clickbutton.set_active(self.project.clickEnabled)
+		self.clickbutton.set_value(value)
 		self.Updating = False
 		
-		if self.project.clickEnabled:
-			tooltip = _("Turn click track off")
-		else:
-			tooltip = _("Turn click track on")
-		
-		self.clicktip.set_tip(self.clickbutton, tooltip, None)
-		
 	#_____________________________________________________________________
 	
 	def OnEditBPM(self, widget, event):
@@ -238,23 +201,20 @@ class TimeLineBar(gtk.Frame):
 			widget -- reserved for GTK callbacks, don't use it explicitly.
 			event -- reserved for GTK callbacks, don't use it explicitly.
 		"""
-		if event.type == gtk.gdk.BUTTON_PRESS:
+		if event.type == gtk.gdk.BUTTON_RELEASE:
 			startWidth = self.headerhbox.size_request()[0]
 			
 			if self.bpmeventbox.parent:
 				self.bpmframe.remove(self.bpmeventbox)
 				self.bpmframe.add(self.bpmedit)
 				self.bpmedit.show()
+				
+				self.bpmedit.grab_add()
 				self.bpmedit.grab_focus()
-			
-			#adjust padding so that the timeline event lanes still line up
-			newWidth = self.headerhbox.size_request()[0]
-			padding = self.alignment.get_padding()
-			self.alignment.set_padding(0, 0, 0, padding[3] - (newWidth -startWidth))
 
 	#_____________________________________________________________________
 	
-	def OnAcceptEditBPM(self, widget=None):
+	def OnAcceptEditBPM(self, widget=None, cancel=False):
 		"""
 		Called when the user finishes editing the beats per minute box.
 		This method then updates the beats per minute value to the value the user 
@@ -263,17 +223,47 @@ class TimeLineBar(gtk.Frame):
 		Parameters:
 			widget -- reserved for GTK callbacks, don't use it explicitly.
 		"""
+		self.bpmedit.grab_remove()
 		
 		if self.bpmedit.parent:
 			self.bpmframe.remove(self.bpmedit)
 			self.bpmframe.add(self.bpmeventbox)
 			self.bpmframe.show_all()
 			
-			newbpm = self.bpmedit.get_value_as_int()
-			self.project.SetBPM(float(newbpm))
-			self.bpmeventbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.bgColor))
+			if not cancel:
+				newbpm = self.bpmedit.get_value_as_int()
+				self.project.SetBPM(float(newbpm))
 		
 	#_____________________________________________________________________
+	
+	def OnEditBPMClick(self, widget, event):
+		"""
+		Handles the button presses while doing a grab on the bpm edit spinbutton.
+		If we get a mouse event, it means the user has clicked outside the spinbutton.
+		So then we save what the user edited and go back to a label.
+		"""
+		if event.type == gtk.gdk.BUTTON_RELEASE:
+			self.OnAcceptEditBPM()
+			return True
+
+	#_____________________________________________________________________
+
+	def OnEditBPMKey(self, widget, event):
+		"""
+		Handles the key presses while editing the instrument name label.
+		Used to make the escape key save the name and then return to normal mode.
+		
+		Parameters:
+			widget -- GTK callback.
+			event -- GTK callback.
+		"""
+		key = gtk.gdk.keyval_name(event.keyval)
+		
+		if key == "Escape":
+			self.OnAcceptEditBPM(cancel=True)
+			return True
+
+	#_____________________________________________________________________
 
 	def OnEditSig(self, widget, event):
 		"""
@@ -331,7 +321,6 @@ class TimeLineBar(gtk.Frame):
 		"""
 		self.project.SetMeter(self.project.meter_nom,
 							  int(combobox.get_active_text()))
-		self.projectview.UpdateSize()
 	
 	#_____________________________________________________________________
 	
@@ -361,10 +350,8 @@ class TimeLineBar(gtk.Frame):
 		
 		if (event.type == gtk.gdk.ENTER_NOTIFY):
 			widget.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.XTERM))
-			self.bpmeventbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.bgActiveColor))
 		else:
 			widget.window.set_cursor(None)
-			self.bpmeventbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.bgColor))
 			
 	#_____________________________________________________________________
 	
@@ -381,20 +368,28 @@ class TimeLineBar(gtk.Frame):
 			return
 		
 		if (event.type == gtk.gdk.ENTER_NOTIFY):
-			self.sigeventbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.bgActiveColor))
+			self.sigeventbox.set_state(gtk.STATE_PRELIGHT)
 		else:
-			self.sigeventbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.bgColor))
+			self.sigeventbox.set_state(gtk.STATE_NORMAL)
 			
 	#_____________________________________________________________________
 
-	def OnClick(self, widget):
+	def OnClickTrackVolume(self, widget, value):
 		"""
 		Called when the click button is clicked. This method will call the project
-		to turn on or shut off the click track.
+		to set the volume of the click track.
 		""" 
 		if not self.Updating:
-			self.project.SetClickEnabled(widget.get_active())
+			self.project.SetClickTrackVolume(value)
 			
 	#_____________________________________________________________________
 	
+	def OnClickButtonEnter(self, widget, event):
+		if event.type == gtk.gdk.ENTER_NOTIFY:
+			self.clickbutton.set_image(self.clickbutton_volume_image)
+		elif event.type == gtk.gdk.LEAVE_NOTIFY:
+			self.clickbutton.set_image(self.clickbutton_metronome_image)
+		
+	#_____________________________________________________________________
+	
 #=========================================================================
